#
#  This is the Plone Help Center FAQ Folder type, with enhanced features
#  like dividing the FAQ into Sections, and Display relevant
#  versions.
#

import urllib

from zope.interface import implements

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from zope.component import getMultiAdapter

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import Products.CMFCore.permissions as CMFCorePermissions
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchemaFolderish, HelpCenterContainerSchema

from Products import ATContentTypes
from PHCFolder import PHCFolder
from Products.PloneHelpCenter.interfaces import IHelpCenterFolder

FAQFolderSchema = HelpCenterBaseSchemaFolderish + Schema((
    TextField('description',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description="Description of the FAQ Container.",
                description_msgid="phc_desc_folder",
                label_msgid="phc_label_folder",
                label="Description",
                i18n_domain = "plonehelpcenter",
                rows=6,)
        ),
    ),) + HelpCenterContainerSchema

class HelpCenterFAQFolder(PHCFolder, ATContentTypes.content.folder.ATFolder):
    """An FAQ Section can hold frequently asked questions with answers."""

    implements(IHelpCenterFolder)

    content_icon = 'faq_icon.gif'

    schema = FAQFolderSchema
    archetype_name = 'FAQ Section'
    meta_type = 'HelpCenterFAQFolder'
    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ('HelpCenterFAQ', )

    security = ClassSecurityInfo()

    typeDescription= 'An FAQ Section can hold frequently asked questions with answers.'
    typeDescMsgId  = 'description_edit_faqfolder'

    # def getTOCSelectOptions(self, current=None):
    #     """
    #     Returns a sequence of dicts:
    #         title: tile of section/page, including numbering
    #         url:   URL of page
    #         current: True if current section/page
    #     This is a convenience for creating an option list.
    #     """
    # 
    #     res = []
    #     for section in getMultiAdapter((self, self.REQUEST,), name="hcf_view").getSectionsToList():
    #         res.append( 
    #             {'title':section, 
    #              'url':'%s/faqsection_view?section=%s' % (self.absolute_url(), urllib.quote(section)), 
    #              'current':False} )
    #     
    #     return res
    #     
    # def getAllPagesURL(self):
    #     return '%s?full=1' % self.absolute_url()

registerType(HelpCenterFAQFolder, PROJECTNAME)



#
# Tests for workflow and permissions in the PHC
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneTestCase.PloneTestCase import default_user

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneHelpCenter.tests import PHCTestCase


class TestWorkflow(PHCTestCase.PHCTestCase):
    """Tests for workflow specific issues in the PHC."""

    def afterSetUp(self):
        PHCTestCase.PHCTestCase.afterSetUp(self)
        pm = self.portal.portal_membership
        pm.addMember( 'test_reviewer', 'pw', ['Member', 'Reviewer'], [] )
        pm.addMember( 'test_manager', 'pw', ['Member', 'Manager'], [] )

    def _publishContent(self, item):
        """Moves content created by the default user to the published state."""
        self.portal.portal_workflow.doActionFor(item, 'submit')
        self.login('test_reviewer')
        self.portal.portal_workflow.doActionFor(item, 'publish')
        self.login(default_user)

    def _checkReviewState(self, item, state):
        """Checks that the content item's review_state is equal to state."""
        itemState = self.portal.portal_workflow.getInfoFor(item, 'review_state')
        self.assertEqual(itemState, state)

    # Next several tests: owners can edit content even after it's published
    
    def testEditPublishedHowto(self):
        newBody = 'Changed to this content while published.'
        howto = self._createHowto(getattr(self.folder.hc, 'how-to'), 'howto1')
        self._publishContent(howto)
        howto.edit(text_format='plain', text=newBody)
        self.assertEqual(howto.getRawText(), newBody)

    def testEditPublishedTutorial(self):
        newDescription = 'New Description.'
        tutorial = self._createTutorial(self.folder.hc.tutorial, 'tut1')
        self._publishContent(tutorial)
        tutorial.edit(description=newDescription)
        self.assertEqual(tutorial.Description(), newDescription)
        tutorial.invokeFactory('HelpCenterTutorialPage', 'newPage')
        page = getattr(tutorial, 'newPage')
        page.setTitle('New Page')
        page.setDescription('A tutorial page added after tutorial was published')
        page.setText=('')
        self.assertEqual(tutorial.newPage.Title(), 'New Page')
        
    def testEditPublishedTutorialPage(self):
        # Edit a page on a tutorial that has been published.
        newDescription = 'New Description.'
        tutorial = self._createTutorial(self.folder.hc.tutorial, 'tut1')
        tutorial.invokeFactory('HelpCenterTutorialPage', 'newPage')
        page = getattr(tutorial, 'newPage')
        page.setTitle('New Page')
        page.setDescription('A tutorial page added after tutorial was published')
        page.setText('')
        self._publishContent(tutorial)
        tutorial.edit(description=newDescription)
        tutorial.newPage.edit(description=newDescription)
        self.assertEqual(tutorial.newPage.Description(), newDescription)

    def testEditPublishedFaq(self):
        newDescription = 'New Description.'
        faq = self._createFAQ(self.folder.hc.faq, 'faq')
        self._publishContent(faq)
        faq.edit(description=newDescription)
        self.assertEqual(faq.Description(), newDescription)

    def testEditPublishedLink(self):
        newUrl = 'http://www.trizpug.org/'
        link = self._createLink(self.folder.hc.link, 'link')
        self._publishContent(link)
        link.edit(url=newUrl)
        self.assertEqual(link.getUrl(), newUrl)

    def testEditPublishedErrorReference(self):
        newBody = 'Changed!'
        errorRef = self._createErrorReference(self.folder.hc.error, 'er')
        self._publishContent(errorRef)
        errorRef.edit(text=newBody)
        self.assertEqual(errorRef.getRawText(), newBody)

    def testEditPublishedDef(self):
        newDefinition = 'Explicit is better than implicit'
        definition = self._createDefinition(self.folder.hc.glossary, 'def')
        self._publishContent(definition)
        definition.edit(description=newDefinition)
        self.assertEqual(definition.Description(), newDefinition)

    # Test a couple of types to make sure that Owners can edit pending
    # and obsolete content, too.

    def testEditPendingHowto(self):
        newBody = 'Changed to this content while published.'
        howto = self._createHowto(getattr(self.folder.hc, 'how-to'), 'howto1')
        self.portal.portal_workflow.doActionFor(howto, 'submit')
        howto.edit(text_format='plain', text=newBody)
        self.assertEqual(howto.getRawText(), newBody)

    def testEditObsoleteHowto(self):
        newBody = 'Changed to this content while published.'
        howto = self._createHowto(getattr(self.folder.hc, 'how-to'), 'howto1')
        self.portal.portal_workflow.doActionFor(howto, 'mark_obsolete')
        howto.edit(text_format='plain', text=newBody)
        self.assertEqual(howto.getRawText(), newBody)
        

    # Next several tests: owners can obsolete their own content at any point

    def testOwnerObsoletesHowto(self):
        howto = self._createHowto(getattr(self.folder.hc, 'how-to'), 'howto1')
        self._publishContent(howto)
        self.portal.portal_workflow.doActionFor(howto, 'mark_obsolete')
        self._checkReviewState(howto, 'obsolete')

    def testOwnerObsoletesTutorial(self):
        tutorial = self._createTutorial(self.folder.hc.tutorial, 'tut1')
        self._publishContent(tutorial)
        self.portal.portal_workflow.doActionFor(tutorial, 'mark_obsolete')
        self._checkReviewState(tutorial, 'obsolete')

    def testOwnerObsoletesFAQ(self):
        faq = self._createFAQ(self.folder.hc.faq, 'faq1')
        self._publishContent(faq)
        self.portal.portal_workflow.doActionFor(faq, 'mark_obsolete')
        self._checkReviewState(faq, 'obsolete')

    def testOwnerObsoletesLink(self):
        link = self._createLink(self.folder.hc.link, 'link1')
        self._publishContent(link)
        self.portal.portal_workflow.doActionFor(link, 'mark_obsolete')
        self._checkReviewState(link, 'obsolete')

    def testOwnerObsoletesErrorReference(self):
        errorRef = self._createErrorReference(self.folder.hc.error, 'er')
        self._publishContent(errorRef)
        self.portal.portal_workflow.doActionFor(errorRef, 'mark_obsolete')
        self._checkReviewState(errorRef, 'obsolete')

    def testOwnerObsoletesDefinition(self):
        definition = self._createDefinition(self.folder.hc.glossary, 'd')
        self._publishContent(definition)
        self.portal.portal_workflow.doActionFor(definition, 'mark_obsolete')
        self._checkReviewState(definition, 'obsolete')

    def testOwnerObsoletesPendingHowto(self):
        howto = self._createHowto(getattr(self.folder.hc, 'how-to'), 'howto1')
        self.portal.portal_workflow.doActionFor(howto, 'submit')
        self.portal.portal_workflow.doActionFor(howto, 'mark_obsolete')
        self._checkReviewState(howto, 'obsolete')

    def testOwnerObsoletesPendingTutorial(self):
        tutorial = self._createTutorial(self.folder.hc.tutorial, 'tut1')
        self.portal.portal_workflow.doActionFor(tutorial, 'submit')
        self.portal.portal_workflow.doActionFor(tutorial, 'mark_obsolete')
        self._checkReviewState(tutorial, 'obsolete')

    def testOwnerObsoletesPendingFAQ(self):
        faq = self._createFAQ(self.folder.hc.faq, 'faq1')
        self.portal.portal_workflow.doActionFor(faq, 'submit')
        self.portal.portal_workflow.doActionFor(faq, 'mark_obsolete')
        self._checkReviewState(faq, 'obsolete')

    def testOwnerObsoletesPendingLink(self):
        link = self._createLink(self.folder.hc.link, 'link1')
        self.portal.portal_workflow.doActionFor(link, 'submit')
        self.portal.portal_workflow.doActionFor(link, 'mark_obsolete')
        self._checkReviewState(link, 'obsolete')

    def testOwnerObsoletesPendingErrorReference(self):
        errorRef = self._createErrorReference(self.folder.hc.error, 'er')
        self.portal.portal_workflow.doActionFor(errorRef, 'submit')
        self.portal.portal_workflow.doActionFor(errorRef, 'mark_obsolete')
        self._checkReviewState(errorRef, 'obsolete')

    def testOwnerObsoletesPendingDefinition(self):
        definition = self._createDefinition(self.folder.hc.glossary, 'd')
        self.portal.portal_workflow.doActionFor(definition, 'submit')
        self.portal.portal_workflow.doActionFor(definition, 'mark_obsolete')
        self._checkReviewState(definition, 'obsolete')

    def testOwnerObsoletesInProgressHowto(self):
        howto = self._createHowto(getattr(self.folder.hc, 'how-to'), 'howto1')
        self.portal.portal_workflow.doActionFor(howto, 'mark_obsolete')
        self._checkReviewState(howto, 'obsolete')

    def testOwnerObsoletesInProgressTutorial(self):
        tutorial = self._createTutorial(self.folder.hc.tutorial, 'tut1')
        self.portal.portal_workflow.doActionFor(tutorial, 'mark_obsolete')
        self._checkReviewState(tutorial, 'obsolete')

    def testOwnerObsoletesInProgressFAQ(self):
        faq = self._createFAQ(self.folder.hc.faq, 'faq1')
        self.portal.portal_workflow.doActionFor(faq, 'mark_obsolete')
        self._checkReviewState(faq, 'obsolete')

    def testOwnerObsoletesInProgressLink(self):
        link = self._createLink(self.folder.hc.link, 'link1')
        self.portal.portal_workflow.doActionFor(link, 'mark_obsolete')
        self._checkReviewState(link, 'obsolete')

    def testOwnerObsoletesInProgressErrorReference(self):
        errorRef = self._createErrorReference(self.folder.hc.error, 'er')
        self.portal.portal_workflow.doActionFor(errorRef, 'mark_obsolete')
        self._checkReviewState(errorRef, 'obsolete')

    def testOwnerObsoletesInProgressDefinition(self):
        definition = self._createDefinition(self.folder.hc.glossary, 'd')
        self.portal.portal_workflow.doActionFor(definition, 'mark_obsolete')
        self._checkReviewState(definition, 'obsolete')

    # Next several tests: reviewers can obsolete any pending content

    def testReviewerObsoletesHowto(self):
        howto = self._createHowto(getattr(self.folder.hc, 'how-to'), 'howto1')
        self._publishContent(howto)
        self.login('test_reviewer')
        self.portal.portal_workflow.doActionFor(howto, 'mark_obsolete')
        self._checkReviewState(howto, 'obsolete')

    def testReviewerObsoletesTutorial(self):
        tutorial = self._createTutorial(self.folder.hc.tutorial, 'tut1')
        self._publishContent(tutorial)
        self.login('test_reviewer')
        self.portal.portal_workflow.doActionFor(tutorial, 'mark_obsolete')
        self._checkReviewState(tutorial, 'obsolete')

    def testReviewerObsoletesFAQ(self):
        faq = self._createFAQ(self.folder.hc.faq, 'faq1')
        self._publishContent(faq)
        self.login('test_reviewer')
        self.portal.portal_workflow.doActionFor(faq, 'mark_obsolete')
        self._checkReviewState(faq, 'obsolete')

    def testReviewerObsoletesLink(self):
        link = self._createLink(self.folder.hc.link, 'link1')
        self._publishContent(link)
        self.login('test_reviewer')
        self.portal.portal_workflow.doActionFor(link, 'mark_obsolete')
        self._checkReviewState(link, 'obsolete')

    def testReviewerObsoletesErrorReference(self):
        errorRef = self._createErrorReference(self.folder.hc.error, 'er')
        self._publishContent(errorRef)
        self.login('test_reviewer')
        self.portal.portal_workflow.doActionFor(errorRef, 'mark_obsolete')
        self._checkReviewState(errorRef, 'obsolete')

    def testReviewerObsoletesDefinition(self):
        definition = self._createDefinition(self.folder.hc.glossary, 'd')
        self._publishContent(definition)
        self.login('test_reviewer')
        self.portal.portal_workflow.doActionFor(definition, 'mark_obsolete')
        self._checkReviewState(definition, 'obsolete')

    # Next several tests: non-reviewers who have the 'Review portal content'
    # permission, such as a manager, can obsolete any pending content
    # XXX: Rather bogus to use a manager here...probably need a normal member
    # who has just that permisson.

    def testManagerObsoletesHowto(self):
        howto = self._createHowto(getattr(self.folder.hc, 'how-to'), 'howto1')
        self._publishContent(howto)
        self.login('test_manager')
        self.portal.portal_workflow.doActionFor(howto, 'mark_obsolete')
        self._checkReviewState(howto, 'obsolete')

    def testManagerObsoletesTutorial(self):
        tutorial = self._createTutorial(self.folder.hc.tutorial, 'tut1')
        self._publishContent(tutorial)
        self.login('test_manager')
        self.portal.portal_workflow.doActionFor(tutorial, 'mark_obsolete')
        self._checkReviewState(tutorial, 'obsolete')

    def testManagerObsoletesFAQ(self):
        faq = self._createFAQ(self.folder.hc.faq, 'faq1')
        self._publishContent(faq)
        self.login('test_manager')
        self.portal.portal_workflow.doActionFor(faq, 'mark_obsolete')
        self._checkReviewState(faq, 'obsolete')

    def testManagerObsoletesLink(self):
        link = self._createLink(self.folder.hc.link, 'link1')
        self._publishContent(link)
        self.login('test_manager')
        self.portal.portal_workflow.doActionFor(link, 'mark_obsolete')
        self._checkReviewState(link, 'obsolete')

    def testManagerObsoletesErrorReference(self):
        errorRef = self._createErrorReference(self.folder.hc.error, 'er')
        self._publishContent(errorRef)
        self.login('test_manager')
        self.portal.portal_workflow.doActionFor(errorRef, 'mark_obsolete')
        self._checkReviewState(errorRef, 'obsolete')

    def testManagerObsoletesDefinition(self):
        definition = self._createDefinition(self.folder.hc.glossary, 'd')
        self._publishContent(definition)
        self.login('test_manager')
        self.portal.portal_workflow.doActionFor(definition, 'mark_obsolete')
        self._checkReviewState(definition, 'obsolete')
    

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestWorkflow))
    return suite

if __name__ == '__main__':
    framework()



from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase

from Products.Five import fiveconfigure
from Products.Five import zcml
from Products.PloneTestCase.layer import onsetup
import Products.PloneHelpCenter
from Products.PloneHelpCenter.config import ADD_CENTER_PERMISSION, \
  ADD_HELP_AREA_PERMISSION

# from Products.Five.testbrowser import Browser

ZopeTestCase.installProduct('PloneHelpCenter')

@onsetup
def setup_product():
    fiveconfigure.debug_mode = True
    zcml.load_config('configure.zcml', Products.PloneHelpCenter)
    fiveconfigure.debug_mode = False

setup_product()
PloneTestCase.setupPloneSite(products=['PloneHelpCenter'])


class PHCTestCase(PloneTestCase.PloneTestCase):

    defaultTitle = 'Default Testing Title'
    defaultVersions = ( 'Version 1.0', 'Version 2.0', 'Different Version1.0', )
    defaultBodyRst = """
    Bogus reST body
    ===============
    
    Here's fake body content for unit tests.
    
    * Looks like a list.
    * Smells like a list.
    * It's a list!
    
    Final content after the list.
    """

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()

    def afterSetUp(self):
        self.portal.manage_permission(ADD_CENTER_PERMISSION,
                                      ['Manager', 'Owner'])
        self.portal.manage_permission(ADD_HELP_AREA_PERMISSION,
                                      ['Manager', 'Owner'])
        self._createHelpCenter(self.folder)

    def _createHelpCenter(self, folder, id='hc', title=defaultTitle, versions=defaultVersions):
        """Creates and returns a refence to a PHC HelpCenter.
        This method publishes a HelpCenter instance under folder.  It fills in
        all of the standard properties."""
        folder.invokeFactory('HelpCenter', id)
        helpCenter = getattr(folder, id)
        helpCenter.setTitle(title)
        helpCenter.setDescription('A HelpCenter instance for unit tests.')
        helpCenter.setVersionsVocab(versions)
        self.portal.portal_workflow.doActionFor(helpCenter, 'submit')
        return helpCenter

    def _createHowto(self, howtoFolder, id, title=defaultTitle):
        """Creates and returns a refence to a PHC Howto.
        This method creates a Howto instance under a folder.  It fills in
        all of the standard properties."""
        howtoFolder.invokeFactory('HelpCenterHowTo', id)
        howto = getattr(howtoFolder, id)
        howto.setTitle(title)
        howto.setDescription('A PHC Howto for unit tests.')
        howto.setText(self.defaultBodyRst) 
        howto.setVersions( ('Version 2.0',) )
        howto.setSections( ('General',) )
        self.portal.plone_utils.editMetadata(howto, format='text/x-rst')
        return howto

    def _createTutorial(self, tutorialFolder, id, title=defaultTitle, numPages=2):
        """Creates and returns a reference to a PHC Tutorial.
        This method creates a Tutorial instance under a folder.  It fills in
        all of the standard properties."""
        tutorialFolder.invokeFactory('HelpCenterTutorial', id)
        tutorial = getattr(tutorialFolder, id)
        tutorial.setTitle(title)
        tutorial.setDescription('A PHC Tutorial for unit tests.')
        tutorial.setVersions( ('Version 2.0',) )
        tutorial.setSections( ('General',) )
        # attach pages
        for i in range(numPages):
            pageNum = i + 1
            id='page%d' % pageNum
            tutorial.invokeFactory('HelpCenterTutorialPage', id)
            newPage = getattr(tutorial, id)
            newPage.setTitle('Test Tutorial Page %d' % pageNum)
            newPage.setDescription('A PHC Tutorial Page (%d) for unit tests.' % pageNum)
            newPage.setText(self.defaultBodyRst)
            self.portal.plone_utils.editMetadata(newPage,format='text/x-rst')
        return tutorial
                                   
    def _createFAQ(self, faqFolder, id, title=defaultTitle):
        """Creates and returns a reference to a PHC FAQ.
        This method creates an FAQ instance under a folder.  It fills in
        all of the standard properties."""
        faqFolder.invokeFactory('HelpCenterFAQ', id=id)
        faq = getattr(faqFolder, id)
        faq.setTitle=(title)
        faq.setDescription('An FAQ for unit tests.  Did you know that this field is supposed to be the questionfaq.set?')
        faq.setText('No one knows; it is one of the great mysteries.')
        faq.setVersions( ('Version 2.0',) )
        faq.setSections( ('General',) )
        self.portal.plone_utils.editMetadata(faq, format='text/plain')
        return faq

    def _createLink(self, linkFolder, id, title=defaultTitle):
        linkFolder.invokeFactory('HelpCenterLink', id)
        link = getattr(linkFolder, id)
        link.setTitle( title )
        link.setDescription( 'A Link for unit tests.' )
        link.setUrl('http://www.plone.org/')
        link.setVersions( ('Version 2.0',) )
        link.setSections( ('General',) )
        return link

    def _createErrorReference(self, errorRefFolder, id, title=defaultTitle):
        errorRefFolder.invokeFactory('HelpCenterErrorReference', id)
        errorRef = getattr(errorRefFolder, id)
        errorRef.setTitle( title )
        errorRef.setDescription( 'An error reference for unit tests.' )
        errorRef.setText( self.defaultBodyRst )
        errorRef.setVersions( ('Version 2.0',) )
        errorRef.setSections( ('General',) )
        self.portal.plone_utils.editMetadata(errorRef, format='text/x-rst')
        return errorRef

    def _createDefinition(self, glossaryFolder, id, title=defaultTitle):
        glossaryFolder.invokeFactory('HelpCenterDefinition', id)
        definition = getattr(glossaryFolder, id)
        definition.setTitle(title)
        definition.setDescription('A definition for unit tests.')
        definition.setVersions( ('Version 2.0',) )
        definition.setSections( ('General',) )
        return definition

    def _createReferenceManual(self, glossaryFolder, id, title=defaultTitle):
        pass

    def _createVideo(self, videoFolder, id, title=defaultTitle):
        pass

class PHCFunctionalTestCase(PloneTestCase.FunctionalTestCase):
    
    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.FunctionalTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()




from zope.interface import implements
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import Products.CMFCore.permissions as CMFCorePermissions

from Products.Archetypes import atapi

from Products.ATContentTypes.content import folder, schemata

from Products.PloneHelpCenter.config import PROJECTNAME
from Products.PloneHelpCenter.interfaces import IHelpCenterFolder
from schemata import HelpCenterContainerSchema
from PHCFolder import PHCFolder

KnowledgeBaseSchema = folder.ATBTreeFolderSchema.copy() + HelpCenterContainerSchema    
schemata.finalizeATCTSchema(KnowledgeBaseSchema, moveDiscussion=False, folderish=True)

class HelpCenterKnowledgeBase(PHCFolder, folder.ATBTreeFolder):
    """
        A Single-Folder KnowledgeBase that can organize a large number of how-tos and tutorials by topic and audience.
    """

    implements(IHelpCenterFolder)

    archetype_name = 'Knowledge Base'
    meta_type = "HelpCenterKnowledgeBase"
    
    schema = KnowledgeBaseSchema

    filter_content_types = 1
    allowed_content_types = ('HelpCenterTutorial', 'HelpCenterHowTo', 'Topic')

    typeDescription= 'A Single-Folder KnowledgeBase that can organize a large number of how-tos and tutorials by topic and audience.'
    typeDescMsgId  = 'description_edit_knowledgebase'

    content_icon = 'helpcenter_icon.gif'

    security = ClassSecurityInfo()


atapi.registerType(HelpCenterKnowledgeBase, PROJECTNAME)



## Script (Python) "updateStaleItems"
##title=Update stale-items portlet, then refresh
##parameters=newInterval

# Stash newInterval (as 'stale_PHC_items_interval' in a session variable,
# then reload the current page
session=context.REQUEST.SESSION
session['stale_PHC_items_interval']=newInterval

return context.REQUEST.RESPONSE.redirect(context.absolute_url())



from zope.interface import implements
from AccessControl import ClassSecurityInfo

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import Products.CMFCore.permissions as CMFCorePermissions
from Products.CMFCore.utils import getToolByName

from Products.CMFPlone.browser.navtree import NavtreeStrategyBase, buildFolderTree

from Products import ATContentTypes
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from schemata import HelpCenterItemSchemaNarrow
from PHCContent import PHCContentMixin
from Products.PloneHelpCenter.config import *
from Products.PloneHelpCenter.interfaces import IHelpCenterNavRoot, IHelpCenterContent

import re
IMG_PATTERN = re.compile(r"""(\<img .*?)src="([^/]+?)"(.*?\>)""", re.IGNORECASE | re.DOTALL)


ReferenceManualSchema = ATContentTypes.content.folder.ATFolderSchema.copy() + HelpCenterItemSchemaNarrow
if GLOBAL_RIGHTS:
    del ReferenceManualSchema['rights']
finalizeATCTSchema(ReferenceManualSchema, folderish=True, moveDiscussion=False)
ReferenceManualSchema['nextPreviousEnabled'].defaultMethod = None  
ReferenceManualSchema['nextPreviousEnabled'].default = True  


class HelpCenterReferenceManual(ATContentTypes.content.folder.ATFolder, PHCContentMixin):
    """A reference manual containing ReferenceManualPages,
    ReferenceManualSections, Files and Images.
    """

    implements(IHelpCenterNavRoot)

    schema = ReferenceManualSchema
    archetype_name = 'Reference Manual'
    security = ClassSecurityInfo()

    typeDescription= 'A Reference Manual can contain Reference Manual Pages and Sections, Images and Files. Index order is decided by the folder order, use the normal up/down arrow in the folder content view to rearrange content.'
    typeDescMsgId  = 'description_edit_referencemanual'


    security.declareProtected(CMFCorePermissions.View,
                                'getReferenceManualDescription')
    def getReferenceManualDescription(self):
        """ Returns the description of the ReferenceManual -- 
        convenience method for ReferenceManualPage
        """
        return self.Description()

    security.declareProtected(CMFCorePermissions.View, 'getTOC')
    def getTOC(self, current=None, root=None):
        """Get the table-of-contents of this manual. 
        
        The parameter 'current' gives the object that is the current page or
        section being viewed. 
        
        The parameter 'root' gives the root of the manual - if not given, this
        ReferenceManual object is used, but you can pass in a 
        ReferenceManualSection instead to root the TOC at this element. The 
        root element itself is not included in the table-of-contents.
        
        The return value is a list of dicts, recursively representing the 
        table-of-contents of this manual. Each element dict contains:
        
            item        -- a catalog brain for the item (a section or page)
            numbering   -- The dotted numbering of this item, e.g. 1.3.2
            depth       -- The depth of the item (0 == top-level item)
            currentItem -- True if this item corresponds to the object 'current'
            children    -- A list of dicts
            
        The list 'children' recursively contains the equivalent dicts for
        children of each section. If the parameter 'current' is not given, no
        element will have current == True.
        """

        if not root:
            root = self

        class Strategy(NavtreeStrategyBase):
            
            rootPath = '/'.join(root.getPhysicalPath())
            showAllParents = False
                
        strategy = Strategy()
        query=  {'path'        : '/'.join(root.getPhysicalPath()),
                 'portal_type' : ('HelpCenterReferenceManualSection',
                                   'HelpCenterReferenceManualPage',),
                 'sort_on'     : 'getObjPositionInParent'}
                
        toc = buildFolderTree(self, current, query, strategy)['children']
        
        def buildNumbering(nodes, base=""):
            idx = 1
            for n in nodes:
                numbering = "%s%d." % (base, idx,)
                n['numbering'] = numbering
                buildNumbering(n['children'], numbering)
                idx += 1
                
        buildNumbering(toc)
        return toc


    security.declareProtected(CMFCorePermissions.View, 'getTOCSelectOptions')
    def getTOCSelectOptions(self, current=None):
        """
        Calls getTOC then cooks the results into a sequence of dicts:
            title: tile of section/page, including numbering
            url:   URL of page
            current: True if current section/page
        This is a convenience for creating an option list.
        """
        
        def doNodes(nodes):
            res = []
            for n in nodes:
                item = n['item']
                res.append( { 
                    'title'   : "%s %s" % (n['numbering'], item.Title),
                    'url'     : item.getURL(),
                    'current' : n['currentItem'],
                } )
                if n['children']:
                    childres = doNodes(n['children'])
                    if childres:
                        res = res + childres
            return res
        
        return doNodes(self.getTOC(current))


    security.declareProtected(CMFCorePermissions.View, 'getTOCInfo')
    def getTOCInfo(self, toc):
        """Get information about a table-of-contents, as returned by getTOC.
        
        The return value is a dict, containing:

            tocList    -- A flat list representing the table-of-contents
            localTOC   -- A toc structure for the contents under the current
                            item (passed in as 'current' to getTOC())
            currentIdx -- The index in tocList of 'current'
            nextIdx    -- The index in tocList of the next item after 'current'
            prevIdx    -- The index in tocList of the next item before 'current'
            parentIdx  -- The index in tocList of the parent of 'current'
            
        The elements 'currentIdx', 'nextIdx', 'prevIdx' and 'parentIdx' may be
        None if either the table-of-contents was not constructed with a current
        item, or if there is no previous/next/parent item. Similarly, 'localTOC' 
        will be None if the table-of-contents was not constructed with a current 
        item.
        
        Each item in the list 'tocList' in the returned dict contains a dict 
        with keys:
        
            item       -- A catalog brain repsenting the item
            numbering  -- The dotted numbering of the item, e.g. 1.3.2.
            depth      -- The depth of the item (0 = top-level item)
            current    -- True if this item represents the current page/section
         
        The parameter 'toc' gives the table of contents, as returned by
        getTOC() above.
        """

        # Let's fake static variables - keep track of what may be our parent
        global parentIdx, prevIdx, prevDepth, prevWasCurrent

        parentIdx = None
        prevIdx = None
        prevDepth = -1
        prevWasCurrent = False

        def addToList(tocInfo, tocItem):
            item      = tocItem['item']
            numbering = tocItem['numbering']
            depth     = tocItem['depth']
            children  = tocItem['children']
            isCurrent = tocItem['currentItem']

            global parentIdx, prevIdx, prevDepth, prevWasCurrent

            numberingList = numbering.split('.')[:-1]
            idxList = [int(number) - 1 for number in numberingList]

            tocInfo['tocList'].append({'item'        : item,
                                       'numbering'   : numbering,
                                       'depth'       : depth,
                                       'currentItem' : isCurrent,
                                       })

            currentIdx = len(tocInfo['tocList']) - 1

            if isCurrent:
                prevWasCurrent = True
                tocInfo['currentIdx'] = currentIdx
                if currentIdx > 0:
                    tocInfo['prevIdx'] = currentIdx - 1
                tocInfo['parentIdx'] = parentIdx
                tocInfo['localTOC'] = tocItem['children']
            elif prevWasCurrent:
                prevWasCurrent = False
                tocInfo['nextIdx'] = currentIdx

            for child in children:
                addToList(tocInfo, child)

            # parent index will be item with depth = depth - 1
            # keep track of potential parents by noting when we move down
            # one step

            if depth > prevDepth:
                parentIdx = prevIdx
                prevDepth = depth

            prevIdx = currentIdx

        tocInfo = {'currentIdx' : None,
                   'nextIdx'    : None,
                   'prevIdx'    : None,
                   'parentIdx'  : None,
                   'tocList'    : [],
                   'localTOC'   : None}

        for topLevel in toc:
            addToList(tocInfo, topLevel)

        return tocInfo


    security.declareProtected(CMFCorePermissions.View, 'addImagePaths')
    def addImagePaths(self, body, baseurl):
        """Fixup image paths in section body"""
        
        # This is a convenience method for use in referencemanual_macros
        # section_collation macro. It looks in body for img tags
        # with relative URLs in the src attribute and prepends the baseurl.
        # TODO: when we not longer need 2.1 compatibility, this belongs in 
        # a view.
                
        return IMG_PATTERN.sub(r"""\1src="%s/\2"\3""" % baseurl, body)


    security.declareProtected(CMFCorePermissions.View, 'referenceManualObject')
    def referenceManualObject(self):
        """ find manual from sub-object """
        return self


    security.declareProtected(CMFCorePermissions.View, 'getAllPagesURL')
    def getAllPagesURL(self):
        """ return URL for all pages view """
        
        return "%s/referencemanual-all-pages" % self.absolute_url()


    security.declareProtected(CMFCorePermissions.View, 'getNextPreviousParentValue')
    def getNextPreviousParentValue(self):
        """ always true """
        return True


    security.declareProtected(CMFCorePermissions.View, 'Rights')
    def Rights(self):
        """ get rights from parent if necessary """
        if self.Schema().has_key('rights'):
            return self.getRawRights()
        else:
            return self.aq_parent.Rights()


    security.declareProtected(CMFCorePermissions.View, 'Creators')
    def Creators(self):
        """ get rights from parent if necessary """
        if self.Schema().has_key('creators'):
            return self.getRawCreators()
        else:
            return self.aq_parent.Creators()


registerType(HelpCenterReferenceManual, PROJECTNAME)




from zope.interface import implements
from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
import Products.CMFCore.permissions as CMFCorePermissions

import Products.CMFCore.permissions as CMFCorePermissions
from Products import ATContentTypes
from Products.ATContentTypes.interfaces import IATDocument

from Products.PloneHelpCenter.config import *
from PHCContent import HideOwnershipFields
from Products.PloneHelpCenter.interfaces import IHelpCenterMultiPage

TutorialPageSchema = ATContentTypes.content.document.ATDocumentSchema.copy()
HideOwnershipFields(TutorialPageSchema)
# Support specified content formats
for key, attr in DEFAULT_CONTENT_TYPES.items():
    setattr(TutorialPageSchema['text'], key, attr)

class HelpCenterTutorialPage(ATContentTypes.content.document.ATDocumentBase):
    """Part of a tutorial."""

    implements(IHelpCenterMultiPage)

    __implements__ = ATContentTypes.content.document.ATDocumentBase.__implements__, IATDocument

    schema = TutorialPageSchema

    archetype_name = 'Page'
    meta_type='HelpCenterTutorialPage'
    content_icon = 'document_icon.gif'

    typeDescription= 'A Tutorial Page contains the text of a single page of the tutorial.'
    typeDescMsgId  = 'description_edit_tutorialpage'

    security = ClassSecurityInfo()

    # Satisfy metadata requirements for items with deleted ownership.
    # It would be great to do this in a mixin or adapter,
    # but the structure of Archetypes prevents that.
    
    security.declareProtected(CMFCorePermissions.View, 'Rights')
    def Rights(self):
        """ get from parent """
        return aq_inner(self).aq_parent.Rights()
    
    security.declareProtected(CMFCorePermissions.View, 'Creators')
    def Creators(self):
        """ get from parent """
        return aq_inner(self).aq_parent.Creators()
    
    security.declareProtected(CMFCorePermissions.View, 'Contributors')
    def Contributors(self):
        """ get from parent """
        return aq_inner(self).aq_parent.Contributors()
        
    security.declareProtected(CMFCorePermissions.View, 'listCreators')
    def listCreators(self):
        """ List Dublin Core Creator elements - resource authors.
        """
        return self.Creators()
        
registerType(HelpCenterTutorialPage, PROJECTNAME)



from AccessControl import allow_module
from Globals import package_home
from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore import utils as CMFCoreUtils
from Products.CMFCore.DirectoryView import registerDirectory
import os, os.path, sys

# Get all the content types in the types directory
from Products.PloneHelpCenter import content

# Ensure that pickles refering to the old 'types' module now find 'content'
sys.modules['Products.PloneHelpCenter.types'] = content

from config import *

registerDirectory(SKINS_DIR, GLOBALS)

# MonkeyPatch CMFDefault.DiscussionItemContainer
import Patch

def initialize(context):

    # Make the utils module importable TTW
    allow_module('Products.PloneHelpCenter.utils')

    contentTypes, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    # Initialise. Note that the ADD_CONTENT_PERMISSION is only used in contained
    # images and files; for our special folders and content types, we have
    # defined a custom factory with either ADD_CENTER_PERMISSION,
    # ADD_HELP_AREA_PERMISSION, ADD_DOCUMENTATION_PERMISSION
    CMFCoreUtils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = contentTypes,
        permission         = ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)

    # Extract constructors for items, areas and the root help center so that
    # they can be given different permissions.
    
    itemConstructors = {}
    areaConstructors = {}
    rootConstructors = {}
    for i in range(0, len(contentTypes)):
        if contentTypes[i].meta_type in ('HelpCenterDefinition', 
                                         'HelpCenterFAQ', 
                                         'HelpCenterTutorial',
                                         'HelpCenterReferenceManual',
                                         'HelpCenterInstructionalVideo',
                                         'HelpCenterLink',
                                         'HelpCenterHowTo',
                                         'HelpCenterErrorReference',):
            itemConstructors[contentTypes[i].meta_type] = constructors[i]
        elif contentTypes[i].meta_type in ('HelpCenterGlossary',
                                           'HelpCenterFAQFolder',
                                           'HelpCenterTutorialFolder',
                                           'HelpCenterReferenceManualFolder',
                                           'HelpCenterInstructionalVideoFolder',
                                           'HelpCenterLinkFolder',
                                           'HelpCenterHowToFolder',
                                           'HelpCenterErrorReferenceFolder',
                                           'HelpCenterKnowledgeBase'):
            areaConstructors[contentTypes[i].meta_type] = constructors[i]
        elif contentTypes[i].meta_type in ('HelpCenter',):
            rootConstructors[contentTypes[i].meta_type] = constructors[i]

    # Set custom add permission for items and the help center root
    for meta_type, constructor in itemConstructors.items():
        context.registerClass(
            meta_type = meta_type,
            constructors = (constructor,),
            permission = ADD_DOCUMENTATION_PERMISSION,
            )
    for meta_type, constructor in areaConstructors.items():
        context.registerClass(
            meta_type = meta_type,
            constructors = (constructor,),
            permission = ADD_HELP_AREA_PERMISSION,
            )
    for meta_type, constructor in rootConstructors.items():
        context.registerClass(
            meta_type = meta_type,
            constructors = (constructor,),
            permission = ADD_CENTER_PERMISSION,
            )




from zope.interface import implements
from AccessControl import ClassSecurityInfo

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import Products.CMFCore.permissions as CMFCorePermissions
from Products import ATContentTypes
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.PloneHelpCenter.config import *
from schemata import HelpCenterItemSchemaNarrow
from PHCContent import PHCContentMixin
from Products.PloneHelpCenter.interfaces import IHelpCenterNavRoot

TutorialSchema = ATContentTypes.content.folder.ATFolderSchema.copy() + HelpCenterItemSchemaNarrow
if GLOBAL_RIGHTS:
    del TutorialSchema['rights']
finalizeATCTSchema(TutorialSchema, folderish=True, moveDiscussion=False)
TutorialSchema['nextPreviousEnabled'].defaultMethod = None  
TutorialSchema['nextPreviousEnabled'].default = True  


class HelpCenterTutorial(ATContentTypes.content.folder.ATFolder, PHCContentMixin):
    """A tutorial containing TutorialPages, Files and Images."""

    implements(IHelpCenterNavRoot)

    schema = TutorialSchema
    archetype_name = 'Tutorial'
    meta_type = portal_type = 'HelpCenterTutorial'
    content_icon = 'tutorial_icon.gif'

    typeDescription= 'A Tutorial can contain Tutorial Pages, Images and Files. Index order is decided by the folder order, use the normal up/down arrow in the folder content view to rearrange content.'
    typeDescMsgId  = 'description_edit_tutorial'

    security = ClassSecurityInfo()


    security.declareProtected(CMFCorePermissions.View, 'getTutorialDescription')
    def getTutorialDescription(self):
        """ Returns the description of the Tutorial--convenience method for TutorialPage """
        return self.Description()


    security.declareProtected(CMFCorePermissions.View, 'getPages')
    def getPages(self, states=[]):
        """Get items"""
        criteria = contentFilter = {'portal_type' : 'HelpCenterTutorialPage'}
        if states:
            criteria['review_state'] = states
        return self.getFolderContents(contentFilter = criteria)


    security.declareProtected(CMFCorePermissions.View, 'getPagePosition')
    def getPagePosition(self, obj, states=[]):
        """Get position in folder of the current context"""
        pages = self.getPages()
        for i in range(len(pages)):
            if pages[i].getId == obj.getId():
                return i
        return None


    security.declareProtected(CMFCorePermissions.View, 'getTOCSelectOptions')
    def getTOCSelectOptions(self, current=None):
        """
        Calls getTOC then cooks the results into a sequence of dicts:
            title: tile of section/page, including numbering
            url:   URL of page
            current: True if current section/page
        This is a convenience for creating an option list.
        """

        res = []
        cid = current.getId()
        for page in self.getPages():
            res.append( {'title':page.Title, 'url':page.getURL, 'current':cid==page.id  } )
        return res


    security.declareProtected(CMFCorePermissions.View, 'getAllPagesURL')
    def getAllPagesURL(self):
        """ return URL for all pages view """

        return "%s/tutorial-all-pages" % self.absolute_url()


        security.declareProtected(CMFCorePermissions.View, 'getNextPreviousParentValue')
        def getNextPreviousParentValue(self):
            """ always true """
            return True


    security.declareProtected(CMFCorePermissions.View, 'Rights')
    def Rights(self):
        """ get rights from parent if necessary """
        if self.Schema().has_key('rights'):
            return self.getRawRights()
        else:
            return self.aq_parent.Rights()


    security.declareProtected(CMFCorePermissions.View, 'Creators')
    def Creators(self):
        """ get rights from parent if necessary """
        if self.Schema().has_key('creators'):
            return self.getRawCreators()
        else:
            return self.aq_parent.Creators()


registerType(HelpCenterTutorial, PROJECTNAME)




## Script (Python) "checkVersionOutdated"
##title=Return 1 if the version is outdated, 0 otherwise
##parameters=versions, currentVersions=None

if not versions:
    return 0
    
if not currentVersions:        
    # Acquire current versions if not given
    currentVersions = context.getCurrentVersions()
        
if not currentVersions:
    return 0

for v in versions:
    if v in currentVersions:
        # Not outdated - we match one of the current versions
        return 0
        
# Outdated - we didn't match anything
return 1


#!/usr/bin/env python

"""
   Usage: sync.py [<product> | <language-code>]
   Note that PYTHON and I18NDUDE must have been set as enviroment variables before calling this script
"""

import os, sys
from utils import getPoFilesAsCmdLine, getPoFilesByLanguageCode, getProduct, getPotFiles, getLongProductName

__PYTHON = os.environ.get('PYTHON', '')
__I18NDUDE = os.environ.get('I18NDUDE', '')

def main():
    if len(sys.argv) == 1:
        print 'You have to specify a product or a language code.'
        sys.exit(1)

    product = getLongProductName(sys.argv[1])
    pot = '%s.pot' % product

    os.chdir('..')

    if not os.path.isfile(pot): # no pot? test for language-code
        poFiles = getPoFilesByLanguageCode(product)
        if poFiles:
            potFiles = getPotFiles()
            for po in poFiles:
                for pot in potFiles:
                    if getProduct(po) == getProduct(pot):
                        os.system(__PYTHON + ' ' + __I18NDUDE + (' sync --pot %s %s') % (pot, po))
        else:
            print 'Neither a pot nor po files for the given argument were found.'
            sys.exit(3)

    else: # product was given
        poFiles = getPoFilesAsCmdLine(product)
        if poFiles == []:
            print 'No po-files were found for the given product.'
            sys.exit(4)

        os.system(__PYTHON + ' ' + __I18NDUDE + (' sync --pot %s %s') % (pot, poFiles))

if __name__ == '__main__':
    main()



"""PloneHelpCenter functional doctests.  This module collects all *.txt
files in the tests directory and runs them. (stolen from Plone)
"""

import os, sys

import glob
import doctest
import unittest
from Globals import package_home
from Products.PloneTestCase import PloneTestCase
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite

from Products.PloneHelpCenter.config import GLOBALS

# Load products
from Products.PloneHelpCenter.tests.PHCTestCase import PHCFunctionalTestCase

OPTIONFLAGS = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

def list_doctests():
    home = package_home(GLOBALS)
    return [filename for filename in
            glob.glob(os.path.sep.join([home, 'tests', '*.txt']))]

def test_suite():

    import Products.Five.testbrowser

    filenames = list_doctests()

    return unittest.TestSuite(
        [Suite(os.path.basename(filename),
               optionflags=OPTIONFLAGS,
               package='Products.PloneHelpCenter.tests',
               test_class=PHCFunctionalTestCase)
         for filename in filenames]
        )



#
# Tests for PHC and the Comment (discussion or talkback) system
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneHelpCenter.tests import PHCTestCase

try:
    from Products.CMFPlone.tests.utils import MockMailHost
except ImportError:
    MockMailHost = None
from Products.PloneHelpCenter.utils import discussion_notify


class TestTutorialPageComments(PHCTestCase.PHCTestCase):
    """Tests related to a bug where comments on objects in a PHC Tutorial's
    items were shoting up on the tutorial itself."""

    def afterSetUp(self):
        PHCTestCase.PHCTestCase.afterSetUp(self)
        self.tf = self.folder.hc.tutorial # tutorial folder
        self.tf.invokeFactory('HelpCenterTutorial', id='t')
        self.tutorial = self.tf.t
        self.tutorial.invokeFactory('HelpCenterTutorialPage', 'page1')
        self.tutorial.invokeFactory('HelpCenterTutorialPage', 'page2')
        self.tutorial.invokeFactory('HelpCenterTutorialPage', 'page3')
        
    def testCommentOnTutorialPage(self):
        title = 'Test comment'
        body = 'head\nbody\nlegs\n'
        discussionTool = self.portal.portal_discussion
        # set up the talkback subobject
        discussionTool.getDiscussionFor(self.tutorial.page2)
        # create a comment on the tutorial page
        self.tutorial.page2.discussion_reply(subject=title, body_text=body,)
        # verify that we can get it back on the page
        talkback = discussionTool.getDiscussionFor(self.tutorial.page2)
        comment = talkback.objectValues()[0]
        self.assertEqual(comment.Title(), title)
        self.assertEqual(comment.EditableBody(), body)
        # verify that the comment doesn't show up on the parent tutorial object 
        talkback = discussionTool.getDiscussionFor(self.tutorial)
        self.assertEqual(talkback.objectValues(), [])
        # verify that the comment doesn't show up on the other tutorial pages
        talkback = discussionTool.getDiscussionFor(self.tutorial.page1)
        self.assertEqual(talkback.objectValues(), [])
        talkback = discussionTool.getDiscussionFor(self.tutorial.page3)
        self.assertEqual(talkback.objectValues(), [])
        
    def testCommentOnTutorialFolder(self):
        title = 'Test folder comment'
        body = 'head\nbody\nlegs\n'
        discussionTool = self.portal.portal_discussion
        # set up the talkback subobject
        discussionTool.getDiscussionFor(self.tutorial)
        # create a comment on the tutorial
        self.tutorial.discussion_reply(subject=title, body_text=body,)
        # verify that we can get it back on the tutorial
        talkback = discussionTool.getDiscussionFor(self.tutorial)
        comment = talkback.objectValues()[0]
        self.assertEqual(comment.Title(), title)
        self.assertEqual(comment.EditableBody(), body)
        # verify that the comment doesn't show up on any of the tutorial pages
        talkback = discussionTool.getDiscussionFor(self.tutorial.page1)
        self.assertEqual(talkback.objectValues(), [])
        talkback = discussionTool.getDiscussionFor(self.tutorial.page2)
        self.assertEqual(talkback.objectValues(), [])
        talkback = discussionTool.getDiscussionFor(self.tutorial.page3)
        self.assertEqual(talkback.objectValues(), [])

class MockMailHostTests(PHCTestCase.PHCTestCase):

    def afterSetUp(self):
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = MockMailHost('MailHost')
        PHCTestCase.PHCTestCase.afterSetUp(self)
        self.tf = self.folder.hc.tutorial # tutorial folder
        self.tf.invokeFactory('HelpCenterTutorial', id='t')
        self.tutorial = self.tf.t
        self.tutorial.invokeFactory('HelpCenterTutorialPage', 'page1')

    def beforeTearDown(self):
        self.portal.MailHost = self.portal._original_MailHost

    def testCommentMailing(self):
        """ Make sure we're mailing comments """

        mailhost = self.portal.MailHost
        self.assertEqual( len(mailhost.messages), 0 )

        # try to notify
        discussion_notify(self.tutorial.page1)
        # there is no sendto address, so we expect no outgoing mail
        self.assertEqual( len(mailhost.messages), 0 )

        # set an owner email address and try again
        owner = self.tutorial.page1.Creator()        
        member = self.portal.portal_membership.getMemberById(owner)
        member.setMemberProperties({'fullname': 'fullname', 'email': 'testuser@testme.com',})
        discussion_notify(self.tutorial.page1)
        self.assertEqual( len(mailhost.messages), 1 )
        
        msg = mailhost.messages[0]
        payload = msg.message.get_payload().decode('base64')
        self.failUnlessEqual( msg.mto[0], 'testuser@testme.com' )
        self.failUnlessEqual( msg.message['subject'], '=?utf-8?q?New_comment_on_page1?=' )
        self.failUnless( payload.find('Someone added a comment on your HelpCenterTutorialPage:\npage1.') > 0 )


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestTutorialPageComments))
    if MockMailHost:
        suite.addTest(makeSuite(MockMailHostTests))
    return suite

if __name__ == '__main__':
    framework()



from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import Products.CMFCore.permissions as CMFCorePermissions

from Products import ATContentTypes
from Products.ATContentTypes.content.document import ATDocumentBase
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.PloneHelpCenter.config import *
from schemata import HelpCenterItemSchemaNarrow
from PHCContent import PHCContentMixin, HideOwnershipFields


FAQSchema = ATContentTypes.content.document.ATDocumentSchema.copy() + HelpCenterItemSchemaNarrow
FAQSchema['description'].widget = \
       TextAreaWidget(
        description = 'More details on the question, if not evident from the title.',
        description_msgid = "help_detailed_question",
        label = "Detailed Question",
        label_msgid = "label_detailed_question",
        rows = 5,
        i18n_domain = "plonehelpcenter",
        )
FAQSchema['text'].widget.label = "Answer"
FAQSchema['text'].widget.label_msgid = "label_answer"
FAQSchema['text'].widget.i18n_domain = "plonehelpcenter"
for key, attr in DEFAULT_CONTENT_TYPES.items():
    setattr(FAQSchema['text'], key, attr)
    
finalizeATCTSchema(FAQSchema, folderish=False, moveDiscussion=False)


class HelpCenterFAQ(ATDocumentBase,PHCContentMixin):
    """A Frequently Asked Question defines a common question with an answer - 
    this is a place to document answers to common questions, not ask them.
    """

    __implements__ = (ATDocumentBase.__implements__,)

    content_icon = 'faq_icon.gif'

    schema = FAQSchema
    archetype_name = 'FAQ'
    meta_type = 'HelpCenterFAQ'

    typeDescription= 'A Frequently Asked Question defines a common question with an answer - this is a place to document answers to common questions, not ask them.'
    typeDescMsgId  = 'description_edit_faq'


registerType(HelpCenterFAQ, PROJECTNAME)



from StringIO import StringIO
from Products.CMFCore.utils import getToolByName

from Products.PloneHelpCenter import config

EXTENSION_PROFILE = 'profile-Products.PloneHelpCenter:default'

## how about the form controller settings?

def install(self):
    out = StringIO()

    tool=getToolByName(self, "portal_setup")

    try:
        tool.runAllImportStepsFromProfile(EXTENSION_PROFILE,
            purge_old=False)
    except AttributeError:   # before plone 3
        old_context = tool.getImportContextID()
        tool.setImportContext(EXTENSION_PROFILE)
        tool.runAllImportSteps(purge_old=False)
        tool.setImportContext(old_context)

    # Add catalog metadata columns and indexes
    catalog = getToolByName(self, 'portal_catalog')
    addCatalogIndex(self, out, catalog, 'isOutdated', 'FieldIndex')
    addCatalogMetadata(self, out, catalog, 'isOutdated')
    addCatalogIndex(self, out, catalog, 'getAudiences', 'KeywordIndex')
    addCatalogMetadata(self, out, catalog, 'getAudiences')
    addCatalogIndex(self, out, catalog, 'getSections', 'KeywordIndex')
    addCatalogMetadata(self, out, catalog, 'getSections')
    addCatalogIndex(self, out, catalog, 'getStartHere', 'FieldIndex')
    addCatalogMetadata(self, out, catalog, 'getStartHere')
    addCatalogIndex(self, out, catalog, 'getVersions', 'KeywordIndex')
    addCatalogMetadata(self, out, catalog, 'getVersions')
    print >> out, "Added PHC items to catalog indexes and metadata"

    print >> out, "Successfully installed %s" % config.PROJECTNAME

    return out.getvalue()


### leaving the old cruft in place for now
### only renaming (re)install not to confuse the quickintaller


from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.PloneHelpCenter.config import *

## from Products.PloneHelpCenter.Extensions import HCWorkflow
## from Products.PloneHelpCenter.Extensions import HCFolderWorkflow
## from Products.PloneHelpCenter.Extensions import HCSubItemWorkflow


def addCatalogIndex(self, out, catalog, index, type, extra = None):
    """Add the given index name, of the given type, to the catalog."""

    if index not in catalog.indexes():
        catalog.addIndex(index, type, extra)
        print >> out, "Added index", index, "to catalog"
    else:
        print >> out, "Index", index, "already in catalog"

def addCatalogMetadata(self, out, catalog, column):
    """Add the given column to the catalog's metadata schema"""
    
    if column not in catalog.schema():
        catalog.addColumn(column)
        print >> out, "Added", column, "to catalog metadata"
    else:
        print >> out, column, "already in catalog metadata"

def removeCatalogIndex(self, out, catalog, index):
    """Delete the given index"""
    
    if index in catalog.indexes():
        catalog.delIndex(index)
        print >> out, "Removed index", index
    else:
        print >> out, "Index", index, "not in catalog"
    
def removeCatalogMetadata(self, out, catalog, column):
    """Delete the given metadata column"""
    
    if column in catalog.schema():
        catalog.delColumn(column)
        print >> out, "Removed column", column
    else:
        print >> out, "Column", column, "not in catalog"

def addToListProperty(self, out, propertySheet, property, value):
    """Add the given value to the list in the given property"""
    current = list(propertySheet.getProperty(property))
    if value not in current:
        current.append(value)
        propertySheet.manage_changeProperties(**{property : current})

    print >> out, "Added %s to %s" % (value, property)


def registerNavigationTreeSettings(self, out):
    """Make the folderish content types not appear in navigation tree.
    We don't want users to think of the HowTo as a folder, even though
    technically, it is."""

    phcTypes = ('HelpCenterHowTo','HelpCenterTutorial','HelpCenterErrorReference',
               'HelpCenterFAQ', 'HelpCenterDefinition', 'HelpCenterLink',
               'HelpCenterInstructionalVideo', 'HelpCenterReferenceManual',
               'HelpCenterTutorialPage', 'HelpCenterReferenceManualPage',
               )
    portalProperties = getToolByName(self, 'portal_properties')
    navtreeProps = getattr(portalProperties, 'navtree_properties')
    
    for t in phcTypes:
        addToListProperty(self, out, navtreeProps, 'metaTypesNotToList', t)
        addToListProperty(self, out, navtreeProps, 'parentMetaTypesNotToQuery', t)

def installPortlets(self, out):
    # prepend to the left_slots list, so it appears on top for Reviewers
    self.left_slots = [ 'here/portlet_stale_items/macros/portlet',] + list(self.left_slots)
    print >>out, "Added portlet"

def registerFormControllerActions(self, out):
    fc_tool = getToolByName(self, 'portal_form_controller')
    fc_tool.addFormAction('content_edit', 'success', 'HelpCenterHowTo', None, 'traverse_to', 'string:edit_reminder')
    fc_tool.addFormAction('content_edit', 'success', 'HelpCenterFAQ', None, 'traverse_to', 'string:edit_reminder')
    fc_tool.addFormAction('content_edit', 'success', 'HelpCenterTutorial', None, 'traverse_to', 'string:edit_reminder')
    fc_tool.addFormAction('content_edit', 'success', 'HelpCenterReferenceManual', None, 'traverse_to', 'string:edit_reminder')
    fc_tool.addFormAction('content_edit', 'success', 'HelpCenterInstructionalVideo', None, 'traverse_to', 'string:edit_reminder')
    fc_tool.addFormAction('content_edit', 'success', 'HelpCenterLink', None, 'traverse_to', 'string:edit_reminder')
    fc_tool.addFormAction('content_edit', 'success', 'HelpCenterErrorReference', None, 'traverse_to', 'string:edit_reminder')
    fc_tool.addFormAction('content_edit', 'success', 'HelpCenterDefinition', None, 'traverse_to', 'string:edit_reminder')
    print >> out, 'Set reminder to publish message hack on objects.'

def installWorkflows(self, out):
    wf_tool = getToolByName(self, 'portal_workflow')

    HCWorkflow.install()
    HCFolderWorkflow.install()
    HCSubItemWorkflow.install()

    if not 'helpcenter_workflow' in wf_tool.objectIds():
        wf_tool.manage_addWorkflow('helpcenter_workflow (Workflow for Help Center Items.)',
                                   'helpcenter_workflow')
    if not 'helpcenterfolder_workflow' in wf_tool.objectIds():
        wf_tool.manage_addWorkflow('helpcenterfolder_workflow (Workflow for Help Center Folders.)',
                                   'helpcenterfolder_workflow')
    if not 'helpcentersubitem_workflow' in wf_tool.objectIds():
        wf_tool.manage_addWorkflow('helpcentersubitem_workflow (Workflow for Help Center Sub-items.)',
                                   'helpcentersubitem_workflow')

    print >> out, 'Installed helpcenter_workflow.'
    print >> out, 'Installed helpcenterfolder_workflow.'
    print >> out, 'Installed helpcentersubitem_workflow'

    wf_tool.setChainForPortalTypes(pt_names=['HelpCenterFAQ'
                                            ,'HelpCenterHowTo'
                                            ,'HelpCenterLink'
                                            ,'HelpCenterTutorial'
                                            ,'HelpCenterReferenceManual'
                                            ,'HelpCenterInstructionalVideo'
                                            ,'HelpCenterErrorReference'
                                            ,'HelpCenterDefinition'], chain='helpcenter_workflow')
    print >> out, 'Set helpcenter_workflow as default for help center content types.'

    wf_tool.setChainForPortalTypes(pt_names=['HelpCenterFAQFolder'
                                            ,'HelpCenterHowToFolder'
                                            ,'HelpCenterLinkFolder'
                                            ,'HelpCenterTutorialFolder'
                                            ,'HelpCenterReferenceManualFolder'
                                            ,'HelpCenterInstructionalVideoFolder'
                                            ,'HelpCenterErrorReferenceFolder'
                                            ,'HelpCenterGlossary'], chain='helpcenterfolder_workflow')
    print >> out, 'Set helpcenterfolder_workflow as default for help center folder types.'

    wf_tool.setChainForPortalTypes(pt_names=['HelpCenterTutorialPage'
                                            ,'HelpCenterReferenceManualSection'
                                            ,'HelpCenterReferenceManualPage'], 
                                    chain='helpcentersubitem_workflow')
    print >> out, 'Set helpcentersubitem_workflow as default for sections and pages.'

def setupPortalFactory(self, out):
    # make new types use portal_factory
    ft = getToolByName(self, 'portal_factory')
    portal_factory_types = ft.getFactoryTypes().keys()
    for t in [
             # With this in the factory, creating the default "how to use this resource" document fails :-(
             # 'HelpCenter',
             'HelpCenterGlossary',
             'HelpCenterDefinition',
             'HelpCenterErrorReference',
             'HelpCenterErrorReferenceFolder',
             'HelpCenterFAQ',
             'HelpCenterFAQFolder',
             'HelpCenterHowTo',
             'HelpCenterHowToFolder',
             'HelpCenterInstructionalVideo',
             'HelpCenterInstructionalVideoFolder',
             'HelpCenterLink',
             'HelpCenterLinkFolder',
             'HelpCenterTutorial',
             'HelpCenterTutorialFolder',
             'HelpCenterTutorialPage',
             'HelpCenterReferenceManual',
             'HelpCenterReferenceManualFolder',
             'HelpCenterReferenceManualSection',
             'HelpCenterReferenceManualPage']:

        if t not in portal_factory_types:
            portal_factory_types.append(t)
            ft.manage_setPortalFactoryTypes(listOfTypeIds=portal_factory_types)

    print >> out, 'New types use portal_factory'

def old_install(self, reinstall=False):
    out = StringIO()

    installTypes(self, out,
                 listTypes(PROJECTNAME),
                 PROJECTNAME)

    from Products.CMFDynamicViewFTI.migrate import migrateFTIs
    migrateFTIs(self, product=PROJECTNAME)

    install_subskin(self, out, GLOBALS)

    registerNavigationTreeSettings(self, out)
    installWorkflows(self, out)
    registerFormControllerActions(self, out)
    setupPortalFactory(self, out)
    
    
    # Add catalog metadata columns and indexes
    catalog = getToolByName(self, 'portal_catalog')
    addCatalogIndex(self, out, catalog, 'isOutdated', 'FieldIndex')
    addCatalogMetadata(self, out, catalog, 'isOutdated')
    print >> out, "Added isOutdated to catalog metadata"

    # Add "stale items" portlet, so HelpCenter Managers and Reviewers can
    # review old stuff to see if it's still useful
    # installPortlets(self, out)
    # removed for now, expensive and should be a separate page or similar,
    # not a portlet. Nice and valuable functionality, though.
    
    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue()

def old_uninstall(self, reinstall=False):
    out = StringIO()

    # remove the stale-items portlet from the portal root object
    #portletPath = 'here/portlet_stale_items/macros/portlet'
    #if portletPath in self.left_slots:
    #    self.left_slots = [p for p in self.left_slots if (p != portletPath)]
    #    print >> out, 'Removed stale-items portlet'

    if not reinstall:
        # Remove catalog metadata columns and indexes
        catalog = getToolByName(self, 'portal_catalog')
        removeCatalogIndex(self, out, catalog, 'isOutdated')
        removeCatalogMetadata(self, out, catalog, 'isOutdated')
        
        removeCatalogIndex(self, out, catalog, 'getAudiences')
        removeCatalogMetadata(self, out, catalog, 'getAudiences')
        
        removeCatalogIndex(self, out, catalog, 'getSections')
        removeCatalogMetadata(self, out, catalog, 'getSections')
        
        removeCatalogIndex(self, out, catalog, 'getStartHere')
        removeCatalogMetadata(self, out, catalog, 'getStartHere')
        
        removeCatalogIndex(self, out, catalog, 'isOutdated')
        removeCatalogMetadata(self, out, catalog, 'isOutdated')
        
        print >> out, "Removed PHC items from catalog indexes and metadata"


    print >> out, "Successfully uninstalled %s." % PROJECTNAME
    return out.getvalue()



#!/usr/bin/env python

"""
   Usage: rebuild-pot.py <product> <path to products skins dir>
   Note that PYTHON and I18NDUDE must have been set as enviroment variables before calling this script

   If you are really lazy you can also use atct, atrbw and at as shorthands and if you set your
   INSTANCE_HOME correct, provide '-i' as second argument. This will automagically use the right version.

   So 'rebuilt-pot.py atrbw -i' is a valid shorthand.
"""

import os, sys, string
from utils import getLongProductName

try:
    import win32api
    WIN32 = True
except ImportError:
    WIN32 = False

__PYTHON = os.environ.get('PYTHON', '')
__I18NDUDE = os.environ.get('I18NDUDE', '')
__INSTANCE_HOME = os.environ.get('INSTANCE_HOME', '')

def main():
    if len(sys.argv) < 3:
        print 'You have to specify the product und skins directory.'
        sys.exit(1)

    product = sys.argv[1]

    product = getLongProductName(product)

    pot = product + '.pot-new'
    manualpot = '%s-manual.pot' % product
    generatedpot = '%s-generated.pot' % product
    log = 'rebuild-%s-pot.log' % product

    domain = product

    os.chdir('..')

    if not os.path.isfile(manualpot):
        print 'Manual pot missing for the given product: %s.' % manualpot
        sys.exit(2)

    skins = sys.argv[2]

    skinserror = False
    if not os.path.isdir(skins):
        if skins == '-i':
            skins = os.path.join(__INSTANCE_HOME, 'Products')
            if os.path.isdir(skins):
                if product == 'plonehelpcenter':
                    skins = os.path.join(skins, 'PloneHelpCenter')
                elif product == 'plone':
                    skins = os.path.join(skins, 'PloneHelpCenter')
                skins = os.path.join(skins, 'skins')
            else:
                skinserror = True
        else:
            skinserror = True

    if skinserror:
        print 'Skins directory (%s) could not be found.' % skins
        sys.exit(3)

    print 'Using %s to build new pot.\n' % skins

    cmd = __PYTHON + ' ' + __I18NDUDE + (' rebuild-pot --pot %s --create %s --merge %s ') % (pot, domain, manualpot)
    cmd += '%s > %s 2>&1' % (skins, log)

    print 'Rebuilding to %s - this takes a while, logging to %s' % (pot, log)
    os.system(cmd)

    # Remove ## X more: occurences
    os.system('sed -r "/## [0-9]+ more:/d" %s > %s2' % (pot, pot))

    # Make paths relative to products skins dir
    step3 = pot
    if WIN32:
        step3 = step3 + '3'
    os.system('sed -r "s,%s,\.,g" %s2 > %s' % (string.replace(skins, '\\', '\\\\'), pot, step3))
    os.remove('%s2' % pot)

    if WIN32:
        # Make directory separator unix like
        os.system('sed -r "/^#:.*/s,\\\\,/,g" %s3 > %s' % (pot, pot))
        os.remove('%s3' % pot)

if __name__ == '__main__':
    main()



import os

def getPoFiles(product, all=False):
    """ Returns all product*.po files in the current folder """
    files = os.listdir(os.curdir)
    if all:
        files = [file for file in files if file.startswith('%s-' % product) and file.endswith('.po')]
    else:
        files = [file for file in files if file.startswith('%s-' % product) and file.endswith('.po') and file != '%s-en.po' % product]
    return files


def getPotFiles(all=False):
    """ Returns all pot files in the current folder
        Normally it doesn't return manual.pots and generated.pots
    """
    files = os.listdir(os.curdir)
    if all:
        files = [f for f in files if f.endswith('.pot')]
    else:
        files = [f for f in files if f.endswith('.pot') and not f[:-4].endswith('manual') and not f[:-4].endswith('generated')]
    return files


def getPoFilesAsCmdLine(product):
    files = getPoFiles(product)
    filestring = ''
    for file in files:
        filestring += file + ' '
    return filestring.rstrip()


def getPotFilesAsCmdLine():
    files = getPotFiles()
    filestring = ''
    for file in files:
        filestring += file + ' '
    return filestring.rstrip()


def getPoFilesByLanguageCode(lang):
    """ Returns all po files which ends with given language code."""
    files = os.listdir(os.curdir)
    files = [file for file in files if file.endswith('.po') and file[:-3].endswith(lang)]
    return files


def getLanguage(product, file):
    """ Returns the language part of a po-file """
    lang = None
    if file.endswith('.po'):
        if file.startswith(product):
            lang = '-'.join(file.split('-')[1:])[:-3]
    return lang


def getProduct(file):
    """ Returns the product part of a file. We assume files to be something like domain-language.po.
        Example: atcontenttypes-pt-br.po
    """
    assert file.endswith('.po') or file.endswith('.pot')

    file = file.split('.')[0] # strip of ending
    file = file.split('-')[0] # only take product

    return file


def getLongProductName(product):
    """ Returns the product name for a known abbreviation or the given value."""
    if product in ['phc']:
        if product == 'phc':
            product = 'plonehelpcenter'
    return product




## Script (Python) "selectRedirect"
##title=Redirector to aid noscript jump widgets
##parameters=target

# only jump within site
if target.startswith(context.portal_url()):
    context.REQUEST.RESPONSE.redirect(target)





#
# Generated by dumpDCWorkflow.py written by Sebastien Bigaret
# Original workflow id/title: folder_workflow/Folder Workflow [Plone]
# Date: 2004/08/11 08:48:26.016 GMT-4
#
# WARNING: this dumps does NOT contain any scripts you might have added to
# the workflow, IT IS YOUR RESPONSABILITY TO MAKE BACKUPS FOR THESE SCRIPTS.
#
# No script detected in this workflow
# 
"""
Programmatically creates a workflow type
"""
__version__ = "$Revision: 1.2 $"[11:-2]

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.PloneHelpCenter.config import *

def setupFolder_workflow(wf):
    "..."
    wf.setProperties(title='Folder Workflow [Plone]')

    for s in ['visible', 'private', 'published']:
        wf.states.addState(s)
    for t in ['retract', 'hide', 'publish', 'show']:
        wf.transitions.addTransition(t)
    for v in ['action', 'review_history', 'actor', 'comments', 'time']:
        wf.variables.addVariable(v)
    for l in ['reviewer_queue']:
        wf.worklists.addWorklist(l)
    for p in ('Add portal content','Access contents information', 
                'Modify portal content', 'View', 'List folder contents',
                ADD_DOCUMENTATION_PERMISSION):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('published')

    ## States initialization
    sdef = wf.states['visible']
    sdef.setProperties(title="""Visible""",
                       transitions=('hide', 'publish'))
    sdef.setPermission('Access contents information', 1, ['Anonymous', 'Manager', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 1, ['Anonymous', 'Manager', 'Reviewer'])
    sdef.setPermission('List folder contents', 0, ['Manager', 'Owner', 'Member'])
    sdef.setPermission('Add portal content', 0, ['Manager', 'Owner',])
    sdef.setPermission(ADD_DOCUMENTATION_PERMISSION, 0, ['Manager', 'Owner',])
    

    sdef = wf.states['private']
    sdef.setProperties(title="""Private""",
                       transitions=('publish', 'show'))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 0, ['Manager', 'Owner'])
    sdef.setPermission('List folder contents', 0, ['Manager', 'Owner'])
    sdef.setPermission('Add portal content', 0, ['Manager', 'Owner',])
    sdef.setPermission(ADD_DOCUMENTATION_PERMISSION, 0, ['Manager', 'Owner',])

    sdef = wf.states['published']
    sdef.setProperties(title="""Open for submissions""",
                       transitions=('hide', 'retract'))
    sdef.setPermission('Access contents information', 1, ['Anonymous', 'Manager'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 1, ['Anonymous', 'Manager'])
    sdef.setPermission('List folder contents', 1, ['Anonymous'])
    sdef.setPermission('Add portal content', 0, ['Manager', 'Owner','Member'])
    sdef.setPermission(ADD_DOCUMENTATION_PERMISSION, 0, ['Manager', 'Owner','Member'])

    ## Transitions initialization
    tdef = wf.transitions['retract']
    tdef.setProperties(title="""Close for submissions""",
                       new_state_id="""visible""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Close for submissions""",
                       actbox_url="""%(content_url)s/content_retract_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Request review'},
                       )

    tdef = wf.transitions['hide']
    tdef.setProperties(title="""Hide""",
                       new_state_id="""private""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Hide""",
                       actbox_url="""%(content_url)s/content_hide_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner'},
                       )

    tdef = wf.transitions['publish']
    tdef.setProperties(title="""Open for submissions""",
                       new_state_id="""published""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Open for submissions""",
                       actbox_url="""%(content_url)s/content_publish_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Modify portal content', 'guard_roles': 'Owner; Manager'},
                       )

    tdef = wf.transitions['show']
    tdef.setProperties(title="""Member makes content visible""",
                       new_state_id="""visible""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Make visible""",
                       actbox_url="""%(content_url)s/content_show_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner'},
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    ## Worklists Initialization
    ldef = wf.worklists['reviewer_queue']
    ldef.setProperties(description="""Reviewer tasks""",
                       actbox_name="""Pending (%(count)d)""",
                       actbox_url="""%(portal_url)s/search?review_state=pending""",
                       actbox_category="""global""",
                       props={'guard_permissions': 'Review portal content', 'var_match_review_state': 'pending'})


def createFolder_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupFolder_workflow(ob)
    return ob

def install():
    addWorkflowFactory(createFolder_workflow,
        id='helpcenterfolder_workflow',
                   title='Workflow for Help Center Folders.')




#
#  This is the Plone Help Center folderish Glossary type, which
#  is a simple container that has Definitions.
#

from zope.interface import implements

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
import Products.CMFCore.permissions as CMFCorePermissions

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchemaFolderish, HelpCenterContainerSchema
from Products import ATContentTypes
from PHCFolder import PHCFolder
from Products.PloneHelpCenter.interfaces import IHelpCenterFolder

GlossarySchema = HelpCenterBaseSchemaFolderish + Schema((
    TextField(
        'description',
        searchable=1,
        required=1,
        primary=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description="Description of the Glossary.",
                description_msgid="phc_desc_folder_glossary",
                label_msgid="phc_label_folder_glossary",
                label="Description",
                i18n_domain = "plonehelpcenter",
                rows= 6,
                )
        ),
    ),) + HelpCenterContainerSchema

class HelpCenterGlossary(PHCFolder, ATContentTypes.content.folder.ATFolder):
    """A Glossary can be used to hold definitions of common terms, listing them 
    in a dictionary-like manner.
    """

    implements(IHelpCenterFolder)

    content_icon = 'glossary_icon.gif'

    schema = GlossarySchema
    archetype_name = 'Glossary'
    meta_type = 'HelpCenterGlossary'
    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ('HelpCenterDefinition', )

    typeDescription= 'A Glossary can be used to hold definitions of common terms, listing them in a dictionary-like manner.'
    typeDescMsgId  = 'description_edit_glossary'

    security = ClassSecurityInfo()

    # aliases = PHCFolder.aliases.copy()
    # aliases.update({'(Default)' : 'glossary_view',
    #                 'view'      : 'glossary_view'})

registerType(HelpCenterGlossary, PROJECTNAME)



"""
 support for the phc_attachements.pt template

 also serves as a 2.5/3.0 compatability layer 
 
"""

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
import Acquisition



class AttachmentsView(BrowserView):
    """ support for howto attachements template """

    def __init__(self, context, request):
        """ set up a few convenience object attributes """
        
        BrowserView.__init__(self, context, request)

        self.catalog = getToolByName(self.context, 'portal_catalog')
        self.actionTool = getToolByName(self.context, 'portal_actions')
        self.portal_url = getToolByName(self.context, 'portal_url')()      
        self.context_path = '/'.join(self.context.getPhysicalPath())

    def folderButtons(self):
        """ valid folder_button actions """
        
        return [button 
                    for button in self.actionTool.listActionInfos(object=Acquisition.aq_inner(self.context))
                        if (button['category'] == 'folder_buttons') and (button['id'] != 'change_state')
                ]


#
# PHC Setup tests
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from AccessControl import Unauthorized
from Products.PloneHelpCenter.tests import PHCTestCase


class TestPortalTypes(PHCTestCase.PHCTestCase):

    def afterSetUp(self):
        self.types = self.portal.portal_types.objectIds()

    def testHelpCenterPortalType(self):
        self.failUnless('HelpCenter' in self.types)

    def testHelpCenterDefinitionType(self):
        self.failUnless('HelpCenterDefinition' in self.types)
        
    def testHelpCenterLinkType(self):
        self.failUnless('HelpCenterLink' in self.types)
        
    def testHelpCenterHowToType(self):
        self.failUnless('HelpCenterHowTo' in self.types)
        
    def testHelpCenterErrorReferenceFolderType(self):
        self.failUnless('HelpCenterErrorReferenceFolder' in self.types)

    def testHelpCenterFaqFolderType(self):
        self.failUnless('HelpCenterFAQFolder' in self.types)

    def testHelpCenterErrorReferenceType(self):
        self.failUnless('HelpCenterErrorReference' in self.types)

    def testHelpCenterTutorialPageType(self):
        self.failUnless('HelpCenterTutorialPage' in self.types)

    def testHelpCenterLinkFolderType(self):
        self.failUnless('HelpCenterLinkFolder' in self.types)

    def testHelpCenterTutorialType(self):
        self.failUnless('HelpCenterTutorial' in self.types)

    def testHelpCenterTutorialFolderType(self):
        self.failUnless('HelpCenterTutorialFolder' in self.types)

    def testHelpCenterVideoType(self):
        self.failUnless('HelpCenterInstructionalVideo' in self.types)

    def testHelpCenterVideoFolderType(self):
        self.failUnless('HelpCenterInstructionalVideoFolder' in self.types)

    def testHelpCenterFaqType(self):
        self.failUnless('HelpCenterFAQ' in self.types)

    def testHelpCenterGlossaryType(self):
        self.failUnless('HelpCenterGlossary' in self.types)

    def testHelpCenterHowToFolderType(self):
        self.failUnless('HelpCenterHowToFolder' in self.types)

    def testHelpCenterKnowledgeBase(self):
        self.failUnless('HelpCenterKnowledgeBase' in self.types)


class TestGlobalAllow(PHCTestCase.PHCTestCase):

    def afterSetUp(self):
        pass

    def testCreateHelpCenter(self):
        # Globally allowed
        PHCTestCase.PHCTestCase.afterSetUp(self)
        self.failUnless('hc' in self.folder.objectIds())

    def typeNotGloballyAllowed(self, type):
        try:
            self.folder.invokeFactory(type, id='h')
        except (ValueError, Unauthorized): # diff'nt errors in 2.0 & 2.1
            return True
        else:
            return False

    def testCreateHelpCenterReferenceManual(self):
        self.failIf(self.typeNotGloballyAllowed('HelpCenterReferenceManual'))

    def testHelpCenterKnowledgeBase(self):
        self.failIf(self.typeNotGloballyAllowed('HelpCenterKnowledgeBase'))        

    def testCreateHelpCenterHowto(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterHowTo'))

    def testHelpCenterDefinition(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterDefinition'))
        
    def testHelpCenterLink(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterLink'))
        
    def testHelpCenterErrorReferenceFolder(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterErrorReferenceFolder'))
        
    def testHelpCenterFaqFolder(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterFAQFolder'))
        
    def testHelpCenterErrorReference(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterErrorReference'))
        
    def testHelpCenterTutorialPage(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterTutorialPage'))
        
    def testHelpCenterLinkFolder(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterLinkFolder'))
        
    def testHelpCenterTutorial(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterTutorial'))
        
    def testHelpCenterTutorialFolder(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterTutorialFolder'))
        
    def testHelpCenterFaq(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterFAQ'))
        
    def testHelpCenterGlossary(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterGlossary'))
        
    def testHelpCenterHowToFolder(self):
        self.failUnless(self.typeNotGloballyAllowed('HelpCenterHowToFolder'))


class _TestFolderishContainmentBase(PHCTestCase.PHCTestCase):
    """A base class for holding a bit of common code"""

    def afterSetUp(self):
        PHCTestCase.PHCTestCase.afterSetUp(self)
        self.hc = self.folder.hc

    def checkAllowedContentTypes(self, folderish, typeIdList):
        """Test that folderish folder allows exactly the types
        whose ids are listed in typeIdList"""
        types = folderish.allowedContentTypes()
        self.assertEqual(len(types), len(typeIdList))
        for item in types:
            self.failUnless(item.getId() in typeIdList)


class TestHelpCenterContainment(_TestFolderishContainmentBase):

    def testHelpCenterAllowedContentTypes(self):
        allowed = [
            'HelpCenterFAQFolder',
            'HelpCenterHowToFolder',
            'HelpCenterTutorialFolder',
            'HelpCenterReferenceManualFolder',
            'HelpCenterInstructionalVideoFolder',
            'HelpCenterLinkFolder',
            'HelpCenterErrorReferenceFolder',
            'HelpCenterGlossary',
            'HelpCenterKnowledgeBase',
            'Folder',
        ]
        self.checkAllowedContentTypes(self.hc, allowed)

    def testHelpCenterPrePopulation(self):
        content = self.hc.objectIds()
        initial = [
            'faq',
            'how-to',
            'tutorial',
            'manual',
            # XXX - per the HelpCenter.py initializeArchetype method "Video type is not yet finished"
            # thus, it should not be found in the self.hc objectIds()
            # 'video',
            'error',
            'link',
            'glossary',
        ]
        self.assertEqual(len(content), len(initial))
        for id in initial:
            self.failUnless(id in content)


class TestHowToFolderContainment(_TestFolderishContainmentBase):

    def afterSetUp(self):
        _TestFolderishContainmentBase.afterSetUp(self)
        self.hf = getattr(self.folder.hc, 'how-to')

    def testHowToFolderAllowedContentTypes(self):
        allowed = [
            'HelpCenterHowTo',
        ]
        self.checkAllowedContentTypes(self.hf, allowed)

    def testCreateHowToInHowToFolder(self):
        # Allowed
        self.hf.invokeFactory('HelpCenterHowTo', id='h')
        self.failUnless('h' in self.hf.objectIds())

    def testCreateDocumentInHowToFolder(self):
        # Not allowed
        self.assertRaises(ValueError, self.hf.invokeFactory, 'Document', id='doc')


class TestHelpCenterErrorReferenceFolderContainment(_TestFolderishContainmentBase):

    def afterSetUp(self):
        _TestFolderishContainmentBase.afterSetUp(self)
        self.hf = self.folder.hc.error

    def testErrorReferenceFolderAllowedContentTypes(self):
        allowed = [
            'HelpCenterErrorReference',
        ]
        self.checkAllowedContentTypes(self.hf, allowed)

    def testCreateErrorReferenceInErrorReferenceFolder(self):
        # Allowed
        self.hf.invokeFactory('HelpCenterErrorReference', id='h')
        self.assertEqual(self.hf.objectIds(), ['h'])
    
    def testCreateDocumentInErrorReferenceFolder(self):
        # Not allowed
        self.assertRaises(ValueError, self.hf.invokeFactory, 'Document', id='doc')


class TestHelpCenterFaqFolderContainment(_TestFolderishContainmentBase):

    def afterSetUp(self):
        _TestFolderishContainmentBase.afterSetUp(self)
        self.hf = self.folder.hc.faq

    def testFaqFolderAllowedContentTypes(self):
        allowed = [
            'HelpCenterFAQ',
        ]
        self.checkAllowedContentTypes(self.hf, allowed)

    def testCreateFaqInFaqFolder(self):
        # Allowed
        self.hf.invokeFactory('HelpCenterFAQ', id='h')
        self.assertEqual(self.hf.objectIds(), ['h'])

    def testCreateDocumentInFaqFolder(self):
        # Not allowed
        self.assertRaises(ValueError, self.hf.invokeFactory, 'Document', id='doc')


class TestHelpCenterLinkFolderContainment(_TestFolderishContainmentBase):

    def afterSetUp(self):
        _TestFolderishContainmentBase.afterSetUp(self)
        self.hf = self.folder.hc.link

    def testLinkFolderAllowedContentTypes(self):
        allowed = [
            'HelpCenterLink',
        ]
        self.checkAllowedContentTypes(self.hf, allowed)

    def testCreateLinkInLinkFolder(self):
        # Allowed
        self.hf.invokeFactory('HelpCenterLink', id='h')
        self.assertEqual(self.hf.objectIds(), ['h'])

    def testCreateDocumentInLinkFolder(self):
        # Not allowed
        self.assertRaises(ValueError, self.hf.invokeFactory, 'Document', id='doc')
    

class TestHelpCenterTutorialFolderContainment(_TestFolderishContainmentBase):

    def afterSetUp(self):
        _TestFolderishContainmentBase.afterSetUp(self)
        self.hf = self.folder.hc.tutorial
        
    def testTutorialFolderAllowedContentTypes(self):
        allowed = [
            'HelpCenterTutorial',
        ]
        self.checkAllowedContentTypes(self.hf, allowed)

    def testCreateTutorialInTutorialFolder(self):
        # Allowed
        self.hf.invokeFactory('HelpCenterTutorial', id='h')
        self.assertEqual(self.hf.objectIds(), ['h'])

    def testCreateDocumentInTutorialFolder(self):
        # Not allowed
        self.assertRaises(ValueError, self.hf.invokeFactory, 'Document', id='doc')
    

class TestHelpCenterTutorialContainment(_TestFolderishContainmentBase):

    def afterSetUp(self):
        _TestFolderishContainmentBase.afterSetUp(self)
        self.hf = self.folder.hc.tutorial
        self.hf.invokeFactory('HelpCenterTutorial', id='tf')
        self.tf = self.hf.tf
        
    def testTutorialFolderAllowedContentTypes(self):
        allowed = [
            'HelpCenterTutorialPage',
            'Image',
            'File',
        ]
        self.checkAllowedContentTypes(self.tf, allowed)

    def testCreateTutorialPageInTutorial(self):
        # Allowed
        self.tf.invokeFactory('HelpCenterTutorialPage', id='h')
        self.assertEqual(self.tf.objectIds(), ['h'])

#    XXX Should we test that we can actually add an Image and a File?

    def testCreateDocumentInTutorial(self):
        # Not allowed
        self.assertRaises(ValueError, self.tf.invokeFactory, 'Document', id='doc')
    

class TestHelpCenterGlossaryContainment(_TestFolderishContainmentBase):

    def afterSetUp(self):
        _TestFolderishContainmentBase.afterSetUp(self)
        self.hf = self.folder.hc.glossary
        
    def testGlossaryAllowedContentTypes(self):
        allowed = [
            'HelpCenterDefinition',
        ]
        self.checkAllowedContentTypes(self.hf, allowed)

    def testCreateDefinitionInGlossaryFolder(self):
        # Allowed
        self.hf.invokeFactory('HelpCenterDefinition', id='h')
        self.assertEqual(self.hf.objectIds(), ['h'])

    def testCreateDocumentInGlossaryFolder(self):
        # Not allowed
        self.assertRaises(ValueError, self.hf.invokeFactory, 'Document', id='doc')


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPortalTypes))
    suite.addTest(makeSuite(TestGlobalAllow))
    suite.addTest(makeSuite(TestHelpCenterContainment))
    suite.addTest(makeSuite(TestHowToFolderContainment))
    suite.addTest(makeSuite(TestHelpCenterErrorReferenceFolderContainment))
    suite.addTest(makeSuite(TestHelpCenterFaqFolderContainment))
    suite.addTest(makeSuite(TestHelpCenterLinkFolderContainment))
    suite.addTest(makeSuite(TestHelpCenterTutorialFolderContainment))
    suite.addTest(makeSuite(TestHelpCenterTutorialContainment))
    suite.addTest(makeSuite(TestHelpCenterGlossaryContainment))
    return suite

if __name__ == '__main__':
    framework()



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchema, GenericHelpCenterItemSchema
from PHCContent import PHCContent
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

InstructionalVideoSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        required=1,
        searchable=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                 description = "Brief explanation of the video's content.",
                 description_msgid = "phc_help_detailed_video",
                 label = "Summary",
                 label_msgid = "phc_label_detailed_video",
                 i18n_domain = "plonehelpcenter"
              ),
        ),
        
    FileField(
        'video_file',
        required=1,
        primary=1,
        widget=FileWidget(
            description="Click 'Browse' to upload a Flash .swf file.",
            description_msgid="phc_help_videofile_description",
            label="Flash File (.swf)",
            label_msgid="phc_label_videofile_description",
            i18n_domain="plonehelpcenter",
            ),
        ),

    ImageField(
        'screenshot',
        required=0,
        sizes=IMAGE_SIZES,
        widget=ImageWidget(
            label='Screenshot',
            label_msgid='phc_label_video_screenshot',
            description='Add a screenshot by clicking the \'Browse\' '
                          'button. Add a screenshot that highlights the '
                          'content of the instructional video.',
            description_msgid='phc_help_video_screenshot',
            i18n_domain='plonehelpcenter',
            ),
        ),

    StringField(
        'duration',
        required=0,
        widget=StringWidget(
            description_msgid='phc_help_video_duration',
            description='Length (in minutes) of the video.',
            label_msgid='phc_label_video_duration',
            label='Duration',
            i18n_domain='plonehelpcenter',
            ),
        ),

    IntegerField(
        'width',
        required=0,
        default='800',
        validators=('isInt',),
        widget=IntegerWidget(
            description_msgid='phc_help_video_width',
            description='Width of the video.',
            label_msgid='phc_label_video_width',
            label='Width',
            i18n_domain='plonehelpcenter',
            ),
        ),

    IntegerField(
        'height',
        required=0,
        default='600',
        validators=('isInt',),
        widget=IntegerWidget(
            description_msgid='phc_help_video_height',
            description='Height of the video.',
            label_msgid='phc_label_video_height',
            label='Height',
            i18n_domain='plonehelpcenter',
            ),
        ),
    ),

    marshall=PrimaryFieldMarshaller(),

    ) + GenericHelpCenterItemSchema

# For some reason, we need to jump through these hoops to get the fields in the
# the right order
InstructionalVideoSchema.moveField('subject', pos='bottom')
InstructionalVideoSchema.moveField('relatedItems', pos='bottom')

class HelpCenterInstructionalVideo(PHCContent,BaseContent):
    """This is an Instructional Video content type, to which you can attach 
    movies and other relevant files.
    """

    content_icon = 'movie_icon.gif'

    schema = InstructionalVideoSchema
    archetype_name = 'Video'
    meta_type = 'HelpCenterInstructionalVideo'
    global_allow = 0
    # allow_discussion = IS_DISCUSSABLE

    typeDescription= 'An Instructional Video can be used to upload Flash instructional videos.'
    typeDescMsgId  = 'description_edit_instructionalvideo'

    # aliases = PHCContent.aliases.copy()
    # aliases.update({'(Default)' : 'ivideo_view',
    #                 'view'      : 'ivideo_view'})

    security = ClassSecurityInfo()

registerType(HelpCenterInstructionalVideo, PROJECTNAME)



from zope.interface import Interface

class IHelpCenter(Interface):
    """ The Help Center itself 
    """

class IHelpCenterFolder(Interface):
    """ Marker interface for PHCFolder class that is
        the ancestory of the PHC containers.
    """

class IHelpCenterContent(Interface):
    """ Marker interface for HelpCenter leaf content classes.
    """

class IHelpCenterHowTo(Interface):
    """ Marker interface for HelpCenterHowTo class.
    """

class IHelpCenterNavRoot(Interface):
    """ Interface for content objects
        that provide nav support for their childeren,
        like manuals.
    """

    def getTOCSelectOptions(current=None):
        pass
        
    def getAllPagesURL():
        pass


class IHelpCenterMultiPage(Interface):
    """ Marker interface for content objects
        that need nav support -- like manual pages.
    """
    


# import things here to make them visible to Plone.
# Note, just commenting something out here isn't enough--
# the Install.py method also should be changed,
# and the __init__ method for HelpCenter.py.

import HelpCenter

import FAQFolder, FAQ

import HowToFolder, HowTo

import TutorialFolder, Tutorial, TutorialPage

import ReferenceManualFolder, ReferenceManual, ReferenceManualSection, ReferenceManualPage

import InstructionalVideoFolder, InstructionalVideo

import LinkFolder, Link

import ErrorReferenceFolder, ErrorReference

import Glossary, Definition

import KnowledgeBase



from zope.interface import implements
from AccessControl import ClassSecurityInfo

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
import Products.CMFCore.permissions as CMFCorePermissions

from Products import ATContentTypes
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.PloneHelpCenter.config import *
from Products.PloneHelpCenter.content.PHCContent import HideOwnershipFields
from Products.PloneHelpCenter.interfaces import IHelpCenterMultiPage


HelpCenterReferenceManualSectionSchema = ATContentTypes.content.folder.ATFolderSchema.copy()
HideOwnershipFields(HelpCenterReferenceManualSectionSchema)


class HelpCenterReferenceManualSection(ATContentTypes.content.folder.ATFolder):
    """A section of a reference manual containing ReferenceManualPages and
    other ReferenceManualSections.
    """

    implements(IHelpCenterMultiPage)
    
    schema = HelpCenterReferenceManualSectionSchema

    archetype_name = 'Section'
    meta_type = 'HelpCenterReferenceManualSection'

    typeDescription= 'A Reference Manual Section can contain Reference Manual Pages, and other Reference Manual (Sub-)Sections. Index order is decided by the folder order, use the normal up/down arrow in the folder content view to rearrange content.'
    typeDescMsgId  = 'description_edit_referencemanualsection'

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.View, 'getSectionDescription')
    def getSectionDescription(self):
        """ Returns the description of the section --
        convenience method for ReferenceManualPage
        """
        return self.Description()

        
    security.declareProtected(CMFCorePermissions.View, 'Rights')
    def Rights(self):
        """ get from parent """
        return self.aq_parent.Rights()
    
    security.declareProtected(CMFCorePermissions.View, 'Creators')
    def Creators(self):
        """ get from parent """
        return self.aq_parent.Creators()
    
    security.declareProtected(CMFCorePermissions.View, 'Contributors')
    def Contributors(self):
        """ get from parent """
        return self.aq_parent.Contributors()
        

registerType(HelpCenterReferenceManualSection, PROJECTNAME)




#
# Tests for FAQ types in the PHC
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneHelpCenter.tests import PHCTestCase

from Products import LinguaPlone

class TestMultilingual(PHCTestCase.PHCTestCase):
    """General tests for multilingual objects."""

    def afterSetUp(self):
        PHCTestCase.PHCTestCase.afterSetUp(self)
        self._createFAQ(self.folder.hc.faq, 'f')
        self.enFaq = self.folder.hc.faq.f
        self.enFaq.addTranslation('fr')
        self.frFaq = self.enFaq.getTranslation('fr')
        
    def testIndependentVersions(self):
        versions = ('1.0','2.0','Strange version')
        self.folder.hc.setVersionsVocab(versions)
        enVersions = self.enFaq.getVersionsVocab()
        frVersions = self.frFaq.getVersionsVocab()
        self.assertEqual(enVersions, frVersions)

    def testUniqueDescription(self):
        enDescription = 'An English description'
        frDescription = 'Not English ;)'
        self.enFaq.setDescription(enDescription)
        self.frFaq.setDescription(frDescription)
        self.failIfEqual(self.enFaq.Description(), self.frFaq.Description())
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMultilingual))
    return suite

if __name__ == '__main__':
    framework()



#!/usr/bin/env python
# encoding: utf-8
"""
upgrades.py

Created by Steve McMahon on 2009-04-17.
"""

from config import PROJECTNAME

from zope.component import getUtility
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import process_types, listTypes


# getPortal and IfInstalled stolen from Collage.
# Thanks, Giles!

def getPortal():
    return getUtility(ISiteRoot)

class NotInstalledComponent(LookupError):
    def __init__(self, cpt_name):
        self.cpt_name = cpt_name
        return

    def __str__(self):
        msg = ("Component '%s' is not installed in this site."
               " You can't run its upgrade steps."
               % self.cpt_name)
        return msg

class IfInstalled(object):
    def __init__(self, prod_name=PROJECTNAME):
        """@param prod_name: as shown in quick installer"""
        self.prod_name = prod_name

    def __call__(self, func):
        """@param func: the decorated function"""
        def wrapper(setuptool):
            qi = getPortal().portal_quickinstaller
            installed_ids = [p['id'] for p in qi.listInstalledProducts()]
            if self.prod_name not in installed_ids:
                raise NotInstalledComponent(self.prod_name)
            return func(setuptool)
        wrapper.__name__ = func.__name__
        wrapper.__dict__.update(func.__dict__)
        wrapper.__doc__ = func.__doc__
        wrapper.__module__ = func.__module__
        return wrapper
        
def migrateBodyTexts(self):

    catalog = getToolByName(self, 'portal_catalog')
    brains = catalog(
        portal_type=['HelpCenterReferenceManualPage',
                     'HelpCenterTutorialPage',
                     'HelpCenterHowTo',
                     'HelpCenterErrorReference',
                     ],
    )

    res = ['Migrate Page Texts ...']
    for obj in [brain.getObject() for brain in brains]:
        body = getattr(obj, 'body', None)
        if body:
            obj.setText(body)
            delattr(obj, 'body')
            res.append(obj.id)

    return "\n".join(res)


def migrateFAQs(self):

    catalog = getToolByName(self, 'portal_catalog')
    brains = catalog(
        portal_type=['HelpCenterFAQ',],
        path='/'.join(self.getPhysicalPath())        
    )

    res = ['Migrate FAQ Answers ...']
    for obj in [brain.getObject() for brain in brains]:
        body = getattr(obj, 'answer', None)
        if body:
            obj.setText(body)
            delattr(obj, 'answer')
            res.append(obj.id)

    return "\n".join(res)


def migrateNextPrev(self):

    catalog = getToolByName(self, 'portal_catalog')
    brains = catalog(
        portal_type=[
            'HelpCenterReferenceManual',
            'HelpCenterReferenceManualSection',
            'HelpCenterTutorial',
            ],
    )

    res = ['Turn on next/prev navigation ...']
    for obj in [brain.getObject() for brain in brains]:
        if not obj.getNextPreviousEnabled():
            obj.setNextPreviousEnabled(True)
            res.append(obj.id)

    return "\n".join(res)


def runTypesUpdate(setuptool):
    """Upgrade types from profile"""

    setuptool.runImportStepFromProfile('profile-Products.PloneHelpCenter:default', 'typeinfo',
                                       run_dependencies=True)


def reindexNearlyAll(portal):
    """
        We need the object_provides index to reflect
        some of our new interfaces.
    """
    
    mytypes = [ t['portal_type'] for t in listTypes(PROJECTNAME) ]
    
    catalog = getToolByName(portal, 'portal_catalog')
    for brain in catalog(portal_type=mytypes):
        brain.getObject().reindexObject('object_provides')


@IfInstalled()        
def runTypesMigration(setuptool):
    """
        Migrate to 3+ types
    """
    
    runTypesUpdate(setuptool)
    
    portal = getPortal()
    print migrateNextPrev(portal)
    print migrateBodyTexts(portal)
    print migrateFAQs(portal)
    
    reindexNearlyAll(setuptool)

    return



from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import Products.CMFCore.permissions as CMFCorePermissions

from Products.PloneHelpCenter.config import *
from HowTo import HowToSchema, HelpCenterHowTo


ErrorReferenceSchema = HowToSchema.copy()
# del ErrorReferenceSchema['versions']
# del ErrorReferenceSchema['sections']
# del ErrorReferenceSchema['audiences']
# del ErrorReferenceSchema['startHere']
ErrorReferenceSchema['text'].widget.description='Explanation of the error.'
ErrorReferenceSchema['text'].widget.description_msgid='phc_help_body_ErrorReference'
ErrorReferenceSchema['text'].widget.i18n_domain = "plonehelpcenter"


class HelpCenterErrorReference(HelpCenterHowTo):
    """An Error Reference can be used to explain a particular error which may 
    arise.
    """ 

    content_icon = 'errorref_icon.gif'

    schema = ErrorReferenceSchema
    archetype_name = 'Error Reference'
    meta_type = 'HelpCenterErrorReference'

    typeDescription= 'An Error Reference can be used to explain a particular error which may arise.'
    typeDescMsgId  = 'description_edit_errorreference'

    security = ClassSecurityInfo()

registerType(HelpCenterErrorReference, PROJECTNAME)



#!/usr/bin/env python
# encoding: utf-8
"""
traversal.py

Created by Steve McMahon on 2009-04-19.
"""

import urllib

from zope.component import adapts, getMultiAdapter
from zope.component import queryMultiAdapter
from zope.app.publisher.browser import getDefaultViewName
from zope.interface import alsoProvides
from zope.publisher.interfaces.http import IHTTPRequest
from ZPublisher.BaseRequest import DefaultPublishTraverse

from interfaces import IHelpCenterFolder, IHelpCenter

class PHCBaseTraverser(DefaultPublishTraverse):

    def publishTraverse(self, request, name):
        # intercept topic

        if name == 'topic':
            furtherPath = request['TraversalRequestNameStack']
            if furtherPath:
                # put argument in request
                request['topic'] = urllib.unquote_plus(furtherPath[0])
                # clean stack
                while furtherPath:
                    furtherPath.pop()
                # use the topic view
                view = getMultiAdapter((self.context, request), name='phc_topic')
                # return view wrapped in context
                return view.__of__(self.context)
            else:
                # bounce back
                return self.context

        return super(PHCBaseTraverser, self).publishTraverse(request, name)


class PHCTraverser(PHCBaseTraverser):
    adapts(IHelpCenter, IHTTPRequest)


class PHCFolderTraverser(PHCBaseTraverser):
    adapts(IHelpCenterFolder, IHTTPRequest)




# This is a zopectl run script to migrate the contents
# of content-type-specific PHC folders into conventional folders
# based on topic.

adminUser = "admin"
myPHC = 'Plone/documentation'
sources = ('how-to', 'tutorial',)
targetId = 'kb'

folderType = 'Folder'

folderIdMap ={
    'Basic Use' : 'basic-use',
    'Configuration and Set-Up' : 'config',
    'Contributing to plone.org' : 'contrib',
    'Developing for Plone' : 'developing',
    'Installation' : 'installation',
    'Internationalization and Localization' : 'i18n',
    'Managing Content' : 'managing-content',
    'Navigation' : 'navigation',
    'Upgrading and Moving' : 'upgrading',
    'Users, Authentication, and Permissions' : 'users-auth',
    'Visual Design' : 'visual-design',
}

import sys

from Testing.makerequest import makerequest

import transaction
from zope.component import queryUtility, queryMultiAdapter
from plone.i18n.normalizer.interfaces import IURLNormalizer
from Products.CMFPlone.utils import _createObjectByType

from AccessControl.SecurityManagement import \
    newSecurityManager, noSecurityManager

from Products.PloneHelpCenter.interfaces import IHelpCenterContent
from Products.PloneHelpCenter.browser.helpcenter import HelpCenterView

normalize = queryUtility(IURLNormalizer).normalize

app = makerequest(app)

acl_users = app.acl_users
user = acl_users.getUser(adminUser)
if user:
    user = user.__of__(acl_users)
    newSecurityManager(None, user)
else:
    print "Retrieving admin user failed"
    sys.exit(1)

phc = app.unrestrictedTraverse(myPHC)
print phc

phcView = HelpCenterView(phc, app.REQUEST)
print phcView

target = phc.unrestrictedTraverse(targetId)
print target

majorTopics = phcView.getMajorTopics()

# create target folders
ids = target.objectIds()
for title in majorTopics:
    id = folderIdMap.get(title, normalize(title))
    if id not in ids:
        print id, title
        _createObjectByType(folderType, target, id)
        obj = target[id]
        obj.setTitle(title)
        obj.reindexObject()
transaction.commit()

# walk the sources
for source in [phc[s] for s in sources]:
    print source
    
    for id, object in source.objectItems():
        # find the first section
        spos = 99
        pSection = ''
        for section in [section.split(':')[0] for section in object.getSections() if section]:
            mypos = majorTopics.index(section)
            if mypos < spos:
                pSection = section
                spos = mypos
        if pSection:
            fid = folderIdMap.get(pSection)
            if fid:
                folder = target[fid]
                if id not in folder.objectIds():
                    # cut and paste -- keeps publications state
                    cb = source.manage_cutObjects(id)
                    folder.manage_pasteObjects(cb)
                    folder[id].reindexObject()
                    transaction.commit()
                else:
                    print "Duplicate id: %s" % id
                    # source.manage_deleteObjects(id)
            else:
                # target folder not present -- should be an odd case
                print "No fid: %s %s" % (id, pSection,)
        else:
            # Leave it where it is
            print "No sections: ",
            print id, object.getSections()
            
noSecurityManager()



""" support for HelpCenter content templates """

from Acquisition import aq_inner

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName

from Products.PloneHelpCenter.interfaces import IHelpCenterNavRoot


class HelpCenterPagedView(BrowserView):
    """ support for HelpCenter paged content templates """
    
    def __init__(self, context, request):
        """ set up a few convenience object attributes """
        BrowserView.__init__(self, context, request)


    def navRootObject(self):
        """ Find the root of the page navigation """

        context = aq_inner(self.context)
        
        parent = context.aq_parent
        while parent and not IHelpCenterNavRoot.providedBy(parent):
            parent = parent.aq_parent
        return parent



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
try:
    import Products.CMFCore.permissions as CMFCorePermissions
except ImportError:
    from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchema, GenericHelpCenterItemSchema
from PHCContent import PHCContent

LinkSchema = HelpCenterBaseSchema + Schema((
    TextField(
        'description',
        default='',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(description_msgid="description_edit_Link",
                description="Description for the Link.",
                label_msgid="description_label_Link",
                label="Description",
                i18n_domain = "plonehelpcenter"),
        ),
    
    StringField(
        'url',
        searchable=1,
        required=1,
        primary=1,
        default='http://',
        languageIndependent=1,
        widget=StringWidget(
                description_msgid='phc_desc_link_url',
                description='Web address.',
                label_msgid='phc_label_link_url',
                label='URL',
                i18n_domain='plonehelpcenter',
                ),
        ),
    ),

    ) + GenericHelpCenterItemSchema

# For some reason, we need to jump through these hoops to get the fields in the
# the right order
LinkSchema.moveField('subject', pos='bottom')
LinkSchema.moveField('relatedItems', pos='bottom')

class HelpCenterLink(PHCContent,BaseContent):
    """A simple archetype"""

    __implements__ = (PHCContent.__implements__,
                      BaseContent.__implements__,)

    content_icon = 'helplink_icon.gif'

    schema = LinkSchema
    archetype_name = 'Link'
    meta_type = 'HelpCenterLink'
    global_allow = 0
    # allow_discussion = IS_DISCUSSABLE

    typeDescription= 'Links are links to other documentation and resources.'
    typeDescMsgId  = 'description_edit_link'

    # aliases = PHCContent.aliases.copy()
    # aliases.update({'(Default)' : 'helplink_view',
    #                 'view'      : 'helplink_view'})


registerType(HelpCenterLink, PROJECTNAME)



from Products.CMFCore.interfaces.Discussions import DiscussionResponse as IDiscussionResponse

def discussion_notify(comment_on_object, variables = {}):
    portal = comment_on_object.portal_url.getPortalObject()

    send_from_address = portal.portal_properties.email_from_address
    send_from_name = portal.portal_properties.email_from_name
    host = portal.plone_utils.getMailHost()
    encoding = portal.plone_utils.getSiteEncoding()
    envelope_from = send_from_address
    
    mt = portal.portal_membership
    if IDiscussionResponse.isImplementedBy(comment_on_object):
        owner = comment_on_object.Creator()
        if owner:
            member = mt.getMemberById(owner)
            if member:
                send_to_address = member.getProperty('email')

                if send_to_address:
                    mail_text = portal.discussion_reply_notify_template(portal, comment_on_object=comment_on_object, send_from_address=send_from_address, send_from_name=send_from_name, send_to_address=send_to_address, **variables)
                    subject = "New comment on " + comment_on_object.title_or_id()

                    # result = host.send(mail_text, send_to_address, envelope_from, subject=subject)
                    result = host.secureSend(mail_text, send_to_address, envelope_from, subject=subject, subtype='plain', charset=encoding, debug=False, From=envelope_from)

        parents = comment_on_object.parentsInThread()
        if not parents:
            return
        comment_on_object = parents[0]
            
    owner = comment_on_object.Creator()
    if owner:
        member = mt.getMemberById(owner)
        if member:
            send_to_address = member.getProperty('email')

            if send_to_address:

                mail_text = portal.discussion_notify_template(portal, comment_on_object=comment_on_object, send_from_address=send_from_address, send_from_name=send_from_name, send_to_address=send_to_address, **variables)
                subject = "New comment on " + comment_on_object.title_or_id()

                # result = host.send(mail_text, send_to_address, envelope_from, subject=subject)
                result = host.secureSend(mail_text, send_to_address, envelope_from, subject=subject, subtype='plain', charset=encoding, debug=False, From=envelope_from)



#
# The Plone How-to container.
#
# The main goals of these containers are to restrict the addable types and
# provide a sensible default view out-of-the-box, like the FAQ view.
#

from zope.interface import implements

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
import Products.CMFCore.permissions as CMFCorePermissions
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HelpCenterBaseSchemaFolderish, HelpCenterContainerSchema

from Products import ATContentTypes
from PHCFolder import PHCFolder
from Products.PloneHelpCenter.interfaces import IHelpCenterFolder

HowToFolderSchema = HelpCenterBaseSchemaFolderish + Schema((
    TextField(
        'description',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description_msgid="phc_desc_howto_folder",
                description="Description for the How-to section.",
                label_msgid="phc_label_howto_folder",
                label="Description",
                i18n_domain = "plonehelpcenter",
                rows=6,
                )
        ),
    )) + HelpCenterContainerSchema

class HelpCenterHowToFolder(PHCFolder, ATContentTypes.content.folder.ATFolder):
    """A How-to Section can contain how-to documents."""

    implements(IHelpCenterFolder)

    content_icon = 'topic_icon.gif'

    schema = HowToFolderSchema
    archetype_name = 'How-to Section'
    meta_type = 'HelpCenterHowToFolder'
    global_allow = 0
    filter_content_types = 1
    allowed_content_types = ('HelpCenterHowTo', )

    typeDescription= 'A How-to Section can contain how-to documents.'
    typeDescMsgId  = 'description_edit_howtofolder'

    security = ClassSecurityInfo()

    # aliases = PHCFolder.aliases.copy()
    # aliases.update({'(Default)' : 'howtofolder_view',
    #                 'view'      : 'howtofolder_view'})

registerType(HelpCenterHowToFolder, PROJECTNAME)



try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
import Products.CMFCore.permissions as CMFCorePermissions
from AccessControl import ClassSecurityInfo
from Products.PloneHelpCenter.config import *
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_hasattr, base_hasattr
from AccessControl import getSecurityManager
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from zope.interface import implements
from Products.PloneHelpCenter.interfaces import IHelpCenterFolder

# Compare section brains by title
def _sectionCmp(a, b):
    # depends on cmp(True, False) == 1
    ash = getattr(a, 'getStartHere', False)
    bsh = getattr(b, 'getStartHere', False)
    if ash == bsh:
        return cmp(a.Title.lower(), b.Title.lower())
    elif ash:
        return -1
    else:
        return 1


class PHCFolder(object):
    """
        Mixin for PHC metadata methods
    """

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.View, 'getAudiencesVocab')
    def getAudiencesVocab(self):
        """Get sections vocabulary."""
        if not safe_hasattr(self.aq_parent, 'getAudiencesVocab'):
            return []
        return self.aq_parent.getAudiencesVocab()


    security.declareProtected(CMFCorePermissions.View, 'getVersionsVocab')
    def getVersionsVocab(self):
        """Get version vocabulary."""
        if not safe_hasattr(self.aq_parent, 'getVersionsVocab'):
            return []
        return self.aq_parent.getVersionsVocab()
            

    security.declareProtected(CMFCorePermissions.View, 'getSectionsVocab')
    def getSectionsVocab(self):
        """Get sections vocabulary"""
        
        localSections = self.sectionsVocab
        if len(localSections):
            return localSections
        elif base_hasattr(self.aq_parent, 'getSectionsVocab'):
            return self.aq_parent.sectionsVocab
        else:
            return ()


    # security.declareProtected(CMFCorePermissions.View, 'Rights')
    # def Rights(self):
    #     """ get rights from parent if necessary """
    #     if self.Schema().has_key('rights'):
    #         return self.getRawRights()
    #     else:
    #         return self.aq_parent.Rights()
    # 
    # 
    # security.declareProtected(CMFCorePermissions.View, 'Contributors')
    # def Contributors(self):
    #     """ get rights from parent if necessary """
    #     if self.Schema().has_key('contributors'):
    #         return self.getContributors()
    #     else:
    #         return self.aq_parent.Contributors()
    # 
    # 
